<GlobalJavascript>
<![CDATA[
  // File: ScriptEngine.js
  // -----------------------------------------------------------------
  function scriptEngineVersion() { return "1.52"; }
  // -----------------------------------------------------------------

  // This file contains the javascript for the GuideMe Script Engine.
  // To create a tease using the script engine see file ScriptTease.xml and
  // follow the instructions in that file.
  //
  // To incorporate the script engine into an existing tease include this file
  // in your tease before the <Pages> section using a command like
  //    <Include file="ScriptEngine.js" />
  // and include file ScriptEngine.xml inside the pages section of your tease.
  //    <Pages>
  //      <Include file="ScriptEngine.xml" />
  //      ... other pages in your tease ...
  //
  // *****************************************************************
  // Developer Notes
  //  This section is only of interest to developers who want to modify
  //  the script engine.
  //
  //  Command processing:
  //    Virtually all command processing is done in the ProcessCommands() function.
  //    The ifSet and ifNotSet commands are preprocessed by the loadScript function
  //    which splits the command into two commands consisting of the "test" and
  //    the command to execute if test is true. The actual processing of those
  //    commands is then handled normally by ProcessCommands().
  //    There are also two special commands handled by getNextPage(). These are
  //    gotoPage and Exit. They are handled differently since they require that
  //    the runScript page exits to some other GuideMe page, so processing of
  //    those commands needs to bubble back up to the pageLoad() function of that page.
  //    The gotoPage command is used internally by the script engine and is not
  //    intended for use in actual scripts. A simple goto command can specify a page.
  //
  //  Special Functions and Keywords
  //    The processing of special functions and keywords like RollTwoDice is
  //    mostly done in function getVariable()
  //
  //    The processing of functions like isSet() and notSet() is done in evaluateExpression()
  // -----------------------------------------------------------------
  function initializeScriptData() {
  // -----------------------------------------------------------------                       //
    // Creates and initializes the object used to pass script data around
    var myName = "initializeScriptData: ";
    var temp;
    var lastEdgeDate;
    // See if scriptData already exists
      var scriptData = scriptVars.get("scriptData");
      if ( scriptData == undefined ) {
        // This is the first script being loaded
        jscriptLog(myName+"Loading initial script..." );
//        comonFunctions.UnsetFlags("UserEdged,UserCame,UserRuined,dontReplaceTags", guide.getFlags());
        comonFunctions.UnsetFlags("dontReplaceTags", guide.getFlags());
        scriptVars.put("EdgeCount","0");

        // Initialize today's edges count or zero it if the date has changed
        temp = parseInt(scriptVars.get("TodaysEdgeCount"));
        if ( temp == undefined || temp == "" ) scriptVars.put("TodaysEdgeCount", "0");

        lastEdgeDate = parseInt(scriptVars.get("LastEdgeDate"));
        if ( isNaN(lastEdgeDate) || lastEdgeDate == undefined ) {
          // Initialize it
          lastEdgeDate = theDate();
          scriptVars.put("LastEdgeDate",""+lastEdgeDate);
          scriptVars.put("TodaysEdgeCount", "0");
          jscriptLog(myName+"Initialized lastEdgeDate to " + lastEdgeDate );
        }
        else {
//          jscriptLog(myName+"lastEdgeDate=" + lastEdgeDate + ", theDate()=" + theDate());
          if ( lastEdgeDate != theDate() ) {
            scriptVars.put("TodaysEdgeCount", "0");
            scriptVars.put("LastEdgeDate",""+theDate());
            jscriptLog(myName+"Updated LastEdgeDate to " + theDate() + " and reset TodaysEdgeCount to 0");
          }
        }


        // Now create a new scriptData copying key settings from the current script
        // The following are inherited by the new script (but the script can change them of course)
          //  TimerStyle
          //  DefaultButton setting and Default button text   // SHOULD THIS INHERIT????
          //  ImageFolder
          //  AudioFolder
          //  VideoFolder
          //  ScriptPath

          // Create the new scriptData
          scriptData = { index: 0, labels: [], globalButtons: [], buttons: [], timer: "normal", metronome: 0, defBtnText: "Continue", includeFiles: [] };

          temp = scriptVars.get("ImageFolder");
          if ( temp == undefined ) scriptData.ImageFolder  = "";
          else scriptData.ImageFolder = temp

          temp = scriptVars.get("AudioFolder");
          if ( temp == undefined ) scriptData.AudioFolder  = "";
          else scriptData.AudioFolder = temp

          temp = scriptVars.get("VideoFolder");
          if ( temp == undefined ) scriptData.VideoFolder  = "";
          else scriptData.VideoFolder = temp

          temp = scriptVars.get("ScriptPath");
          if ( temp == undefined ) scriptData.ScriptPath  = "";
          else scriptData.ScriptPath = temp

          temp = scriptVars.get("onScriptExit");
          if ( temp == undefined ) scriptData.onScriptExit  = "";
          else scriptData.onScriptExit = temp

          //  Save current state of flags.
          scriptData.AutoLineBreaks = isSet("AutoLineBreaks");
          scriptData.TextOnLeft = isSet("TextOnLeft");
          scriptData.CardGameLayout1 = isSet("CardGameLayout1");
          scriptData.CardGameLayout2 = isSet("CardGameLayout2");
          scriptData.KeepDelays = isSet("KeepDelays");
          scriptData.KeepMetronomes = isSet("KeepMetronomes");
          scriptData.dontReplaceTags = isSet("dontReplaceTags");

        // Set default flag state
        comonFunctions.UnsetFlags("AutoLineBreaks,TextOnLeft,CardGameLayout1,CardGameLayout2,KeepDelays,KeepMetronomes,dontReplaceTags", guide.getFlags());

        scriptVars.put("scriptData",scriptData);
        return scriptData;
      }
      else {
        // A script is already running. Save settings and push it onto the stack.
        jscriptLog(myName+"Calling secondary script..." );
        var scriptStack = scriptVars.get("scriptDataStack");
        if ( scriptStack == undefined ) scriptStack = [];

        // Push the current scriptdata onto the stack and save it
        scriptStack.push(scriptData);
        scriptVars.put("scriptDataStack",scriptStack);

        // Now create a new scriptData copying key settings from the current script
        // The following are inherited by the new script (but the script can change them of course)
          //  TimerStyle
          //  DefaultButton setting and Default button text   // SHOULD THIS INHERIT????
          //  ImageFolder
          //  AudioFolder
          //  VideoFolder
          //  ScriptPath

        var newScriptData = { index: 0, labels: [], globalButtons: [], buttons: [], timer: scriptData.timer, metronome: 0, defBtnText: scriptData.defBtnText, includeFiles: []};

        // Save current settings so we can restore them later
          temp = scriptVars.get("ImageFolder");
          if ( temp == undefined ) newScriptData.ImageFolder  = "";
          else newScriptData.ImageFolder = temp

          temp = scriptVars.get("AudioFolder");
          if ( temp == undefined ) newScriptData.AudioFolder  = "";
          else newScriptData.AudioFolder = temp

          temp = scriptVars.get("VideoFolder");
          if ( temp == undefined ) newScriptData.VideoFolder  = "";
          else newScriptData.VideoFolder = temp

          temp = scriptVars.get("ScriptPath");
          if ( temp == undefined ) newScriptData.ScriptPath  = "";
          else newScriptData.ScriptPath = temp

          temp = scriptVars.get("onScriptExit");
          if ( temp == undefined ) newScriptData.onScriptExit  = "";
          else newScriptData.onScriptExit = temp

        //  Save current state of flags for the active script:
          newScriptData.AutoLineBreaks = isSet("AutoLineBreaks");
          newScriptData.TextOnLeft = isSet("TextOnLeft");
          newScriptData.CardGameLayout1 = isSet("CardGameLayout1");
          newScriptData.CardGameLayout2 = isSet("CardGameLayout2");
          newScriptData.KeepDelays = isSet("KeepDelays");
          newScriptData.KeepMetronomes = isSet("KeepMetronomes");
          newScriptData.dontReplaceTags = isSet("dontReplaceTags");

        // Set standard defaults
        scriptVars.put("onScriptExit","");

        // Set default flag state
        comonFunctions.UnsetFlags("AutoLineBreaks,TextOnLeft,CardGameLayout1,CardGameLayout2,KeepDelays,KeepMetronomes,dontReplaceTags", guide.getFlags());

        scriptVars.remove("scriptData");            // Out with the old
        scriptVars.put("scriptData",newScriptData)  // In with the new
        return newScriptData;
      }
  }

  // -----------------------------------------------------------------
  function loadScript( file, startLabel  ) {
  // -----------------------------------------------------------------
    // loadScript loads the specified script file, performing some initial
    // preprocessing and handling some low level commands like "Include"
    // It also defines two special scripts to handle reporting errors and
    // gotoPage commands.
    // The optional startLabel parameter lets you start the script at a specific
    // label instead of at the beginning of the script.
    //
    var myName = "loadScript: ";
    jscriptLog(myName+"Initializing... " );
    var temp;
    var label;
    var j;
    var k;
    var found;
    var Debug = true;

    scriptData = initializeScriptData();

    script = [];

    // Insert the error display script and add it to the label index
    scriptData.labels[scriptData.labels.length] = ["displayerror", script.length]; // Record the label and it's script index
    script[script.length] = "displayError:";
    script[script.length] = "set dontReplaceTags";
    script[script.length] = "unset TextOnLeft,CardGameLayout1,CardGameLayout2";
    script[script.length] = "Button OK";
    scriptData.errorPageIndex = script.length;
    script[script.length] = "/Common/Oops.png, ";   // Append error text here
    script[script.length] = "unset dontReplaceTags";
    script[script.length] = "exit";

    // Now we're where the actual user script will start
    scriptData.labels[scriptData.labels.length] = ["startscript", script.length]; // Record the label and it's script index
    var scriptStartIndex = script.length;     // Record this for later use in reporting errors
                                              // Since we just added a bunch of lines to the beginning of
                                              // the script, line X no longer refers to line X
                                              // of the users script, but to line X of our modified script.
                                              // Recording this lets us report a number closer to the users
                                              // actual line number. It will still be off because comments
                                              // throw the number off too, but at least it's better.
    script[script.length] = "StartScript:";
    var startAt = "StartScript";  // After loading this will always be either
                                  // StartScript if all was successful, or displayError
                                  // if any errors are detected.

    // Initialize scriptData to this point so we can report errors if needed.
    scriptData.script = script;
    scriptVars.put("scriptData", scriptData);

    // And now we're finally ready to try loading the script

    jscriptLog(myName+"Loading " + file );

    temp = findScriptFile(file);
    if ( temp == "" ) {
      reportScriptError("Script file \"" + file + "\" not found." );
      return; // Couldn't find the file.
    }
    else {
      // Record the filename of the script in the script data
//      scriptData.filename = file;
      scriptData.filename = temp; // Changed Aug 19/17
    }
//    jscriptLog("Found scriptfile " + temp);

    var fileText = "";
    filetext = comonFunctions.jsReadFile(temp);
//    try {
//      jscriptLog("loadScript: filetext.length()=" + filetext.length());
//    }
//    catch(err) {
//      jscriptLog("loadScript: filetext.length=" + filetext.length);
//    }
    var fileArray = filetext.match(/^.*(\r\n|$)/gm);
    jscriptLog(myName+"lines read=" + fileArray.length );

    var inTextBlock = false;
    jscriptLog(myName+"PreProcessing script... ");
    for(var i=0;i<fileArray.length;i++) { // Let's run through the file and pre-process things
      var line = fileArray[i];
      line = line.replace(/[\n\r]/g,"");    // Strip newline and carriage return characters
      line = line.replace(/^\s+|\s+$/g,""); // Trim leading and trailing whitespace
      line = line.replace(/\s+/g," ");      // replace multiple spaces with a single space
//      jscriptLog(myName+"line[" + i + "]='" + line + "'");
      // Drop blank lines and comments
      if ( line == "" || line.substr(0,2) == "//" ) continue; // It's blank or a comment so skip this line
      if ( line.search(/\s+\/\//) != -1 ) {            // Line has a trailing comment
        line = line.substr(0,line.search(/\/\//));  // Strip the trailing comment
        line = line.replace(/\s+$/,"");             // And Trim trailing whitespace again
      }

      // So we have a line that isn't blank and isn't a comment.

      // Run it through validateTags to prevent tag errors from messing us up during execution
      // If it's okay then we check for some special commands
      var temp = validateTags(line);
      if ( temp != "" ) {
        // Tag validation reported errors
        startAt = "displayError";
        script[scriptData.errorPageIndex] += "Error: Script line " + (i+1) + " : " + temp;
      }
      else if (! inTextBlock && line.search(/^\S+:$/) != -1 ) { // This is a label
        // Make sure the label is not a duplicate, add it to the script, and
        // add it to the label index while we're at it.
//        label = line.substring(0,line.length-1);  // Trim the colon
//        label = label.toLowerCase(); // Force it to lower case
//        // Now make sure it's not a duplicate
//        var isDup = false;
//        for ( var j=0;j<scriptData.labels.length;j++) {
//          if (scriptData.labels[j][0] == label.toLowerCase() ) {
//            isDup = true;
//            break;
//          }
//        }
        label = line.substring(0,line.length-1);  // Trim the colon
        temp = label.toLowerCase(); // Force it to lower case
        // Now make sure it's not a duplicate
        var isDup = false;
        for ( var j=0;j<scriptData.labels.length;j++) {
          if (scriptData.labels[j][0] == temp ) {
            isDup = true;
            break;
          }
        }
        if ( isDup ) {
          // It's a duplicate so this is an error
          script[scriptData.errorPageIndex] += "Error: Duplicate label \"" + line + "\" at script line " + (i+1) + "\n";
          startAt = "displayError";
        }
        else {
          // Add it to the script and to the index.
//          jscriptLog(myName+"Adding label '" + label + "' with target index " + script.length);
//          scriptData.labels[scriptData.labels.length] = [label, script.length]; // Record the label and it's script index
          // Record the label and it's script index. The label is recorded both in
          // all lowercase form and in whatever case it was specified in the script.
          // The latter is used by ScriptLabelsMenu so it's consistent for the user.
          // Internally we use the all lower case form
          scriptData.labels[scriptData.labels.length] = [temp, script.length, label];
          script[script.length] = line;
        }
      }
      else if (! inTextBlock && line.search(/^include\s+/i) != -1 ) {
        // Found an Include command.
        var includeFile;
        var found;
        var incFile = line.substr(8);  // Trim off command - everything else is filename
        incFile = incFile.replace(/^\"|\"\s*$/g,"");  // Strip off quotes
        jscriptLog(myName+"Include " + incFile );

        includeFile = findScriptFile(incFile);
        jscriptLog(myName+"Got " + includeFile );
        if ( includeFile == "" ) {
            script[scriptData.errorPageIndex] += "Error: Include file " + incFile + " not found at script line " + (i+1) + "\n";
            startAt = "displayError";
        }
        else {
          // Has this file already been included?
          found = false;
          for (temp=0;temp<scriptData.includeFiles.length;temp++ ) {
            // jscriptLog(myName+ "Checking " + includeFile + " against " + scriptData.includeFiles[temp]);
            if ( scriptData.includeFiles[temp] == includeFile ) {
              found = true;
              break;
            }
          }
          // If it's not been included yet, include it
          if ( found ) {
            // The file has already been included
            jscriptLog(myName+ "File " + includeFile + " already included");
          }
          else {
            jscriptLog(myName+"Including " + includeFile );
            scriptData.includeFiles[scriptData.includeFiles.length] = includeFile;
            filetext = comonFunctions.jsReadFile(includeFile);
            var incFileArray = filetext.match(/^.*([\n\r]+|$)/gm);
            jscriptLog(myName+"Loaded " + incFileArray.length + " lines."  );
            fileArray = fileArray.slice(0,i).concat(incFileArray).concat(fileArray.slice(i+1));
            i = i - 1; // The first line of the included file replaces the Include command
                       // So we subtract one to reprocess this index.
          }
        }
      }
//      else if (! inTextBlock && line.search(/^if[not]*set/i) != -1 ) {
      else if (! inTextBlock && line.search(/^ifset|^ifnotset/i) != -1 ) {
        // Found an ifset or ifnotset command. These are split into two commands.
        // The first is the ifSet or ifNotSet portion, and the second is the
        // command to be executed if the condition is true.
        // The script processing then finds the ifSet/ifNotSet command and tests the
        // condition. If the condition is true the next script command is executed,
        // otherwise the next script command is skipped. Neat eh?
//        jscriptLog(myName+"Found an ifset or ifnotset command: " + line);
        line = line.replace(/\s*\|\s*/g,"|");     // Strip spaces around a vertical bar (Or operator)
//        line = line.replace(/\s*,\s*/g,",");    // Strip spaces around commas // Removed October 2016 since the command portion can contain commas and this was messing them up
        line = line.replace(/\s*\+\s*/g,"+");     // Strip spaces around plus signs (+)

//        jscriptLog(myName+"Testing command " + line);
//        var cmdParts = line.match(/^(if[notset]*\s+\S+\s*)\s+(.*)$/i);
        var cmdParts = line.match(/^(if[notse]*[\s\(]+\S+[\s\)]+)(.*)$/i);

        if ( cmdParts == undefined ) {
            jscriptLog(myName+"*** Failed to parse ifSet or ifNotSet command");
            script[scriptData.errorPageIndex] += "Error: Invalid ifSet or ifNotSet command found at line " + (i+1) + "\n&nbsp;&nbsp;" + line +"\n";
            startAt = "displayError";
        }
        else {
//          // Dump the identified parts for debugging and testing
//          for ( var j=0;j<cmdParts.length;j++ ) {
//            jscriptLog(myName+"CmdParts[" + j + "]=" + cmdParts[j]);
//          }
          if ( cmdParts.length == 3) {  // We're expecting 3 parts
            // cmdPart[0] is the full text of the line
            // cmdPart[1] is the "ifSet flags" or "ifNotSet flags" portion
            // cmdPart[2] is the command to execute if the condition is true
            cmdParts[1] = cmdParts[1].replace(/[\(\)]/g," "); // Replace optional brackets with space
            script[script.length] = cmdParts[1]; // Add the if command to the script.
            script[script.length] = cmdParts[2]; // And add the command portion to the script
          }
          else {
            // This is an error
            script[scriptData.errorPageIndex] += "Error: Invalid ifSet or ifNotSet command found at script line " + (i+1) + "\n&nbsp;&nbsp;" + line +"\n";
            startAt = "displayError";
          }
        }
      }
      else if (! inTextBlock && line.search(/^if[\s\(]+/i) != -1 ) { // Tests for the word if followed by a space or opening bracket
        // Found an if command.
        // An if command structure is
        //  if condition command
        // The condition must be enclosed in parentheses (these round brackets)
        // These are split into two commands. The first part is the condition
        // to be evaluated, and the second is the command to be executed if the
        // condition is true.
        // During script processing we find the if command and test the
        // condition. If the condition is true the next script command is executed,
        // otherwise the next script command is skipped. Neat eh?
//        jscriptLog(myName+"Found an if command: " + line);

        // Find the first opening bracket
        j = 0;
        while ( j < line.length && line.substr(j,1) != "(" ) j++;
        if ( j >= line.length ) { // No brackets found
            // This is an error
            script[scriptData.errorPageIndex] += "Error: Missing brackets in If command at script line " + (i+1) + "\n&nbsp;&nbsp;" + line +"\n";
            startAt = "displayError";
        }
        else {
          k = 1;  // Count bracket level
          j = j + 1;  // Skip the opening bracket
          // Scan through counting opening and closing brackets until they balance
          while ( j < line.length && k > 0 ) {
            if ( line.substr(j,1) == "(" ) k++;
            if ( line.substr(j,1) == ")" ) k--;
            j++;
          }

          // line.substr(j) should now be pointing to the final bracket of the condition
          // or possibly to the end of the string if  someone messed up
          if ( j >= line.length ) {
              // This is an error
              script[scriptData.errorPageIndex] += "Error: Unbalanced brackets in If command at script line " + (i+1) + "\n&nbsp;&nbsp;" + line +"\n";
              startAt = "displayError";
          }
          else {
            // Found the closing bracket
            j++;  // Character following the closing bracket
            script[script.length] = line.substring(0,j); // Add the "if condition" part to the script.
//            jscriptLog(myName+"If condition is \"" + script[script.length-1] + "\"" );

            k = script.length;  // Save the next script index
            script[script.length] = line.substr(j);      // And add the command portion to the script
            script[k] = script[k].replace(/^\s+/g,""); // Trim leading whitespace
//            jscriptLog(myName+"   Command is \"" + script[k] + "\"" );
          }
        }
      }
      else {
        // Not a special command so just add it to the script

        // Are we starting a text block?
        if ( line.search(/\s*{|{\s*$/) != -1 ) { // Whitespace followed by openeing brace or last non-whitespace character is opening brace
          inTextBlock = true;
//          jscriptLog(myName+"Starting text block.");
        }
        // Are we ending a text block?
        if ( line.search(/}\s*$/) != -1 ) {       // Last non-whitespace character is a closing brace
          inTextBlock = false;
//          jscriptLog(myName+"End of text block.");
        }

        script[script.length] = line;
      }
    }
    jscriptLog(myName+"Finished PreProcessing... ");
    scriptData.script = script;
    // And start the script
    if ( startAt ==  "displayError") {
      // Errors were detected on load
      scriptData.index = 0;
    }
    else {
      if ( startLabel == undefined || startLabel == "" ) scriptData.index = scriptStartIndex; // Start at the top
      else {
        // Start at the specified label
        scriptData.index = findScriptLabel(startLabel);
        if ( scriptData.index == -1 ) {
          // Specified start label not found
          scriptData.index = 0;
          script[scriptData.errorPageIndex] += "Error: Specified starting label not found: " + startLabel + "\n";
        }
      }
    }
    jscriptLog(myName+"Script load complete. Found " + scriptData.script.length + " commands, Script start index=" + scriptData.index);
    scriptVars.put("scriptData", scriptData);
    return;
  }

  // ----------------------------------------------------------------------
  function findScriptFile(file,path) {
    // Search the ScriptPath or the given path for the requested script file and return the full path

    var myName = "findScriptFile: ";
    jscriptLog(myName+"Looking for script file " + file);
    // Locates the script file and returns the resulting path
    // Do our best to find the file.
    var temp;
    var result = "";
    var folders;
    var f = 0;
    var found = false;

    // If the filename begins with a slash try the filename as specified in case
    // we have a full path specified in an include or call command like
    //  include /Teases/MyTease.scr
    //    or
    //  Call /Teases/AllDay/AllDay1.scr
//    if (file.charAt(0) == "/" && comonFunctions.fileExists(file) == true ) {
    if (file.charAt(0) == "/" ) {
      temp = matchFile(file);
      if( temp != "" ) {
//      jscriptLog(myName+"Checking " + file);
        found = true;
        result = temp;
      }
    }

    if ( ! found ) {
      // If not found try the path or script path
      if ( path != undefined ) folders = path;
      else folders = "" + scriptVars.get("ScriptPath");
//    folders = scriptVars.get("ScriptPath");
//      jscriptLog(myName+"Type of folders is " + typeof(folders));
//      jscriptLog(myName+"folders = " + folders);
      if ( folders == undefined || folders == "" || folders == "null" ) {
        folders = ["Scripts"];  // An array of 1
        jscriptLog(myName+"Using default script path = " + folders);
      }
      else {
        folders = folders.split(/,\s*/);
      }
      // Loop through the folders in ScriptPath one at a time
      while ( f < folders.length && ! found ) {
        if ( folders[f] != undefined && folders[f] != "" ) {
            temp = folders[f] + "/" + file;
            jscriptLog(myName+"Checking (1) " + temp);
//            if (comonFunctions.fileExists(temp) == true ) {
            temp = matchFile(temp);
            if (temp != "") {
              found = true;
              result = temp;
            }
        }
        f++;
      }
    }
    // If the file is still not found and does NOT begin with a slash try the
    // filename as specified in case we have a simple setup where the user did not
    // set a script path, and all script files are just in the media folder or something
    if ( ! found ) {
      jscriptLog(myName+"Checking (2) " + file);
//      if ( file.charAt(0) != "/" && comonFunctions.fileExists(file) == true ) {
      if ( file.charAt(0) != "/" ) {
        temp = matchFile(file);
        if ( temp != "" ) {
          found = true;
          result = temp;
        }
      }
    }

    jscriptLog(myName+" result is \"" + result + "\"");
    return result;
  }
  // ----------------------------------------------------------------------
  function ListAllScripts(path,trimExtension,trimPath) {

    // Returns a list of all scripts found on the script path, or on the path given
    // in the optional path argument.
    // The return value is an array of filenames, optionally including the path and extension.
    // This function is used by the GuidemeScripts and RunRandomScript pages to get the list
    // of available scripts to choose from.
    // The optional path argument is a comma separated list of folders to search.
    // If the argument is omitted or is null, the default is to search the path
    // specified in the ScriptPath variable, or if no ScriptPath is set to search
    // the Scripts folder, and the media folder.
    //
    //  If the optional trimExtension argument is true (default) the .scr extension is
    //  trimmed from the filename for display purposes. If the argument is false
    //  the extension is kept and you will get entries like "Script1.scr" instead of just "Script1"
    //
    //  If the optional trimPath argument is true (default) the path portion is trimmed
    //  from the filenames to make it more suitable for display purposes.
    //  If the argument is false the path is kept and you get entries like "/Scripts/Script1.scr"
    //  instead of just "Script1.scr".
    //

    var myName = "ListAllScripts(): ";
    jscriptLog(myName+"Looking for script files");
    // Locates the script file and returns the resulting path
    // Do our best to find the file.
    var temp;
    var scriptFiles = [];
    var folders;
    var f = 0;
    var i;

    if ( path != undefined ) folders = "" + path;
    else folders = "" + scriptVars.get("ScriptPath");
    jscriptLog(myName+"scriptPath = " + folders);

    if ( trimExtension == undefined ) trimExtension = true;
    if ( trimPath == undefined ) trimPath = true;

//    jscriptLog(myName+"typeof scriptPath = " + typeof(folders));
//    if ( ! folders  ) jscriptLog(myName+"scriptPath is not defined" );
//    if ( folders == undefined ) jscriptLog(myName+"scriptPath is undefined" );
//    if ( folders == ""  ) jscriptLog(myName+"scriptPath is null string" );
//    if ( folders == "null"  ) jscriptLog(myName+"scriptPath is the string \"null\"" );
//    if ( folders == "undefined" ) jscriptLog(myName+"scriptPath is the string \"undefined\"" );

    if ( folders == undefined || folders == "" || folders == "null") {
      folders = ["Scripts"];  // An array of 1
//      jscriptLog(myName+"Using default path " );
    }
    else {
      folders = folders.split(/,\s*/);
      jscriptLog(myName+"After splitting folders = " + folders);
    }
    // Loop through the folders in ScriptPath one at a time
    for ( f=0; f<folders.length; f++) {
      jscriptLog(myName+"Checking folder \"" + folders[f] + "\"");
      if ( folders[f] != undefined && folders[f] != "" ) {
          temp = listScripts(folders[f], ! trimPath);
//    if ( ! temp  ) jscriptLog(myName+"temp is not defined" );
//    if ( temp == undefined ) jscriptLog(myName+"temp is undefined" );
//    if ( temp == ""  ) jscriptLog(myName+"temp is null string" );
//    if ( temp == "null"  ) jscriptLog(myName+"temp is the string \"null\"" );
          jscriptLog("listAllScripts: Found " + temp.length + " scripts");
          if ( temp.length > 0 ) {
            // Add the files to the list
            for (i=0;i<temp.length;i++) {
              // Remove the .scr extension for display purposes
              if ( trimExtension ) temp[i] = temp[i].substr(0,temp[i].length-4);
              // If a script of this name is not already in the list, add it to the list
              // If a script of the same name is already in the list we ignore this one.
              if ( scriptFiles.indexOf(temp[i]) == -1  ) scriptFiles[scriptFiles.length] = temp[i];
              else jscriptLog(myName+"  Script " + folders[f] + "/" + temp[i].toString() + " ignored. Duplicate script name.");
            }
          }
      }
    }
//    jscriptLog("listAllScripts: Found a total of " + scriptFiles.length + " scripts");
//    for ( i=0; i<scriptFiles.length; i++ ) jscriptLog("listAllScripts: script[" + i + "] = " + scriptFiles[i]);
    return scriptFiles;
  }
  // ----------------------------------------------------------------------
  function listScripts(scriptFolder,includePath) {
      // Get the list of script files in the specified folder
      // By default only the filename portion of scripts is returned and the folder
      // is omitted. If the optional includePath argument is true the folder name is included
      // in the returned values so you get something like "/Scripts/Script1.scr" instead of just "Script1.scr"
      jscriptLog("listScripts: Listing scripts in " + scriptFolder);
      var i;
      var j;
      var scriptFiles;
      var filesArray;
//            comonFunctions.ListFiles crashes if folder does not exist
      if ( comonFunctions.directoryExists(scriptFolder) == false ) {
        scriptFiles = [];
//        jscriptLog("listScripts: Folder " + scriptFolder + " not found");
      }
      else {
        scriptFiles = "" + comonFunctions.ListFiles(scriptFolder);   // Is ListFiles() new in Guideme 3.3 ?????
//        jscriptLog("listScripts: ListFiles returned " + scriptFiles);
        if ( includePath == undefined ) includePath = false;
        if ( scriptFiles != "" ) {
          filesArray = scriptFiles.split(",");
//          jscriptLog("listScripts: Found " + filesArray.length + " files");
          // Filter out only *.scr files
          scriptFiles = [];
          for(i=0; i<filesArray.length; i++) {
//              jscriptLog("listScripts: Checking " + filesArray[i] );
              if ( filesArray[i].search(/\.scr$/i) != -1 ) {
                // file has a .scr extension so keep it
                if ( includePath ) scriptFiles[scriptFiles.length] = "/" + scriptFolder + "/" + filesArray[i];
                else scriptFiles[scriptFiles.length] = filesArray[i];
//                jscriptLog("listScripts: [" + (scriptFiles.length-1) + "] = " + scriptFiles[scriptFiles.length-1]);
              }
              else {
                jscriptLog("listScripts: " + filesArray[i] + " ignored");
              }
          }
        }
      }
//      jscriptLog("listScripts: Returning " + scriptFiles.length + " scripts");
      return scriptFiles;
  }
  // ----------------------------------------------------------------------
  function findScriptLabel( label ) {
    // Find the label in the script. Returns the script index of the label,
    // or -1 if the label is not found.
    var foundIt = -1;
    scriptData = scriptVars.get("scriptData");
//    jscriptLog("findScriptLabel: Index contains " + scriptData.labels.length + " labels");
    if ( label.substr(label.length-1) == ":" ) { // chop the colon if it was specified by accident
      label = label.substr(0,label.length-1);
    }

    if ( scriptData != "" && scriptData != undefined ) {
      label = label.toLowerCase();
      for ( var i=0;i<scriptData.labels.length;i++) {
//        jscriptLog("findScriptLabel: Comparing \"" + scriptData.labels[i][0] + "\" with \"" + label + "\"");
        if (scriptData.labels[i][0] == label ) {
          foundIt = scriptData.labels[i][1];  // Return the index
          break;  // Exit the loop and return
        }
      }
    }
//    if ( foundIt == -1 ) jscriptLog("findScriptLabel: Label not found!");
//    else jscriptLog("findScriptLabel: Found it!");
    return foundIt;
  }
  // ----------------------------------------------------------------------
  function gotoTarget(target) {
    // Go to the specified target, which can be a script label or a Guideme page
//    target = replaceTags(target); // Handle tag replacements for commands like goto Label<RandomFrom 1,10>
//    if ( target == undefined ) return;  // replaceTags encountered a fatal error
//    jscriptLog("gotoTarget: After replacing tags target is " + target);

    if ( target != "" ) {
      var newIndex = findScriptLabel(target);
      if ( newIndex != -1 ) {
//      if ( findScriptLabel(target) != -1 ) {
//        jscriptLog("gotoTarget: Found label " + target + " at index " + newIndex);
//        gotoLabel(target);
        scriptData = scriptVars.get("scriptData");  // Get the scriptData
        scriptData.index = newIndex;
        scriptVars.put("scriptData", scriptData);   // And update the data
      }
      else {
        var pageExists = guide.getChapters().get("default").getPages().containsKey(target);
        if ( pageExists ) { // A guideme page of the given name exists so go there
          // Going to a guideme page is always a gosub operation so goto page and gosub page are identical
          goSubTarget(target);
        }
        else { // Error - target not found
          reportScriptError("gotoTarget: No Label or Page named " + target + " was found.");
        }
      }
    }
  }
  // ----------------------------------------------------------------------
  function goSubTarget(target) {
    // Go to the specified target, but prepare for coming back when we hit
    // a return command. Target can be a script label or a Guideme page.
    // goSubTarget assumes that the current script index is pointing to the next
    // command to be executed, rather than the current command that initiated
    // the gosub operation.

    if ( target != "" ) {
      var pageExists = guide.getChapters().get("default").getPages().containsKey(target);

      if ( pageExists || findScriptLabel(target) != -1 ) {
        // We have a match for the target so let's do it.

        scriptData = scriptVars.get("scriptData");  // Get the scriptData

        // And sort out where we're going
        var newIndex = findScriptLabel(target);
        if ( newIndex != -1 ) {
          // If going to a script label we push the return point onto the stack,
          // and we expect there will be an explicit "return" command to return
          // from the subroutine
          jscriptLog("goSubTarget: Found script label " + target);
          if ( ! scriptData.hasOwnProperty("returnStack") ) scriptData.returnStack = []; // Initialize the stack if it doesn't exist
          scriptData.returnStack.push(scriptData.index); // Push the return point on to the stack
          jscriptLog("goSubTarget: Current script index is " + scriptData.index + ", Command " + scriptData.script[scriptData.index] + ", New index = " + newIndex);
          scriptData.index = newIndex;
          scriptVars.put("scriptData",scriptData);
        }
        else { // A guideme page of the given name exists so go there
          jscriptLog("goSubTarget: Found page named " + target);
          // When going to a Guideme page the next command is always the next one in sequence
          // but there will be no explicit return command. The Guideme page or pages just return
          // control to "runScript" when they are finished
          // Here we just set scriptData.GotoPage = true and set scriptData.text to the page name
          // and then ShowScriptPage will handle the rest
          scriptData.GotoPage = true;
          scriptData.text = target;
        }
        scriptVars.put("scriptData", scriptData); // And update the data
      }
      else { // Error - target not found
        reportScriptError("goSubTarget: No Label or Page named " + target + " was found.");
      }
    }
  }

  // ----------------------------------------------------------------------
  function reportScriptError(text) {
//    jscriptLog("ScriptError: " + text);
    scriptData = scriptVars.get("scriptData");
    scriptData.script[scriptData.errorPageIndex] = "/Common/Oops.png, " + text + "\n";
    scriptData.globalButtons = [];  // Remove any global buttons that were defined
    scriptVars.put("scriptData", scriptData);
    gotoTarget("displayError"); // Right now this is always index 0, so really, we could take a shortcut
  }

  // ----------------------------------------------------------------------
  function exitScript() {
    // Exit the current script. This function can be used by Buttons or Delays
    // in guideme pages to cleanly exit the running script. It performs the same
    // actions as an "exit" command in a script.
    // Use it like this:
    //  <Button target="runScript" onclick="exitScript()">Quit</Button>
    //    OR
    //  <Delay target="runScript" onTriggered="exitScript()" />

    var scriptData = scriptVars.get("scriptData");  // Get the scriptData
    if ( scriptData != undefined && scriptData != "" ) {
      scriptData.isSpecialCommand = true;
      scriptData.text = "exit";
    }
  }
  // ----------------------------------------------------------------------
  function returnToGosub() {
    // Return to the previous gosub
    var myName = "returnToGosub: ";
    var scriptData = scriptVars.get("scriptData");
    if ( scriptData != undefined ) {
        if ( scriptData.hasOwnProperty("returnStack") && scriptData.returnStack.length > 0 ) {
          // We have return points on the stack so do it
          scriptData.index = scriptData.returnStack.pop(); // Pop the most recent return point from the stack
          jscriptLog(myName+"Returning to script index=\"" + scriptData.index + "\"");
          scriptVars.put("scriptData", scriptData);
        }
        else {
          // Hit a return statement but there was no goSub first. See if we got here from a button.
          // Treat this like an "exit" command to support Call/return or
          // gosub/return cases, but log a message just in case.
          jscriptLog(myName+"Found a return command without a gosub. Defaulting to \"exit\"");
          exitScript();
        }
    }
  }
  // ----------------------------------------------------------------------
  function onButtonClick(btnText, target) {
    if ( target == undefined ) target = "";
    target = target.replace(/^\s+|\s+$/g,""); // Trim leading and trailing spaces
    var scriptData = scriptVars.get("scriptData");  // Get the scriptData
    jscriptLog("onButtonClick: Starting... btnText=\"" + btnText +"\", target=\"" + target + "\"");
    // All buttons execute this function when they are clicked. The parameters are
    // the button text and the button target.

    // Look for special buttons for automatically counting edges, setting flags etc.

    var btnText = btnText.toLowerCase();

    // The following search tests several patterns as follows (not case sensitive)
    //  ^[i\s]*edge[d]*\s*.*$   // The full button text is "edge", "edged", "I edged" or "I edged " followed by anything
    //  ^edging$                // The button text is "edging"
    //  ^on the edge$           // The button text is the phrase "on the edge"
    if ( btnText.search(/^[i\s]*edge[d]*\s*.*$|^edging$|^on the edge$/i) != -1 ) {
      // Found a button that indicates the user edged.
      //  Examples: Edged, I Edged, I edged Mistress, Edging, On the edge
      jscriptLog("onButtonClick: Found an \"Edged\" button (" + btnText + ")" );
      SECountEdge();
    }
    // The search pattern here supports "Came" or "I came", "I came" followed by
    // anything as in "I came Mistress", "I came master" or "I camelot". Sewriously!
    else if (btnText.search(/^came$|^i came.*$/i) != -1 ) {
//      jscriptLog("onButtonClick: Found a \"Came\" button (" + btnText + ")" );
      comonFunctions.SetFlags("UserCame", guide.getFlags());
    }
    // The search pattern here supports "Ruined", "I ruined" or "I ruined" followed
    // by anything as in "I ruined Mistress", "I ruined master" or "I ruined my lunch". Yup!
    // It will NOT match "Ruin" since that's ambiguous and you could be asking
    // "What kind of orgasm are you allowed"
    // Button Full
    // Button Ruin
    //
    else if (btnText.search(/^ruined$|^i ruined.*$/i) != -1 ) {
//      jscriptLog("onButtonClick: Found a \"Ruined\" button (" + btnText + ")" );
      comonFunctions.SetFlags("UserRuined", guide.getFlags());
    }

    if ( target != "" ) {
      var lower = target.toLowerCase();
//      jscriptLog("onButtonClick: Transferring to target=\"" + target + "\"");
      var newIndex = findScriptLabel(target);
      // Check for special button handling. Normally when we have a button
      // that specifies a target it's a "goto" operation. However, the target can
      // be prefaced with "gosub " to change that behaviour.
      // We also check for some special purpose commands.

      if ( lower == "return" ) {       // Return to the last gosub
        jscriptLog("onButtonClick: return command found " );
        returnToGosub();
      }
      else if ( lower == "exit" ) {    // Exit the script
        jscriptLog("onButtonClick: Exit command found");
          exitScript();
      }
      else if ( newIndex != -1 ) {
        // Found a script label so goto the specified label
        jscriptLog("onButtonClick: Goto script label " + target);
        scriptData.index = newIndex;
        scriptVars.put("scriptData", scriptData);   // And update the data
      }
      else {
        jscriptLog("onButtonClick: Checking for a page name or gosub in " + target);
        var words = target.split(/\s+/);  // Split the command using spaces

        var isGuidemePage = false;
        if ( guide.getChapters().get("default").getPages().containsKey(words[0]) ) isGuidemePage = true;

        if ( isGuidemePage ) {
          jscriptLog("onButtonClick: Found Guideme page " + words[0]);
          goSubTarget(words[0]);
        }
        else if ( target.search(/^\s*gosub /i) != -1 ) {
          if ( words.length > 1 ) {
            jscriptLog("onButtonClick: Found gosub to " + words[1] );
            goSubTarget(words[1]);
          }
          else {
            jscriptLog("onButtonClick: Found a gosub but failed to parse target in command \"" + target + "\"" );
          }
        }
        else {
          // Try to execute it and see if it's some other command. If not it's an error.
          if ( executeCommand(target) == false ) {
            reportScriptError("onButtonClick: No Label or Page named " + target + " was found.");
          }
        }
      }
    }
  }


  // ----------------------------------------------------------------------
  function findButton(btnText, buttonList) {
      // If the specified button exists in the given list return it's index, otherwise return -1
      btnText = btnText.toLowerCase();
      for (var i=0;i<buttonList.length;i++) {
        if ( buttonList[i][0].toLowerCase() == btnText ) return i;
      }
      return -1;
  }

  // ----------------------------------------------------------------------
  function findStyle(styleName) {
    // Find the named style in the list of known styles and return it's index, otherwise return -1
    styleName = styleName.toLowerCase();
    if ( scriptData.hasOwnProperty("Styles") ) {
      for (var i=0;i<scriptData.Styles.length;i++) {
        if ( scriptData.Styles[i][0].toLowerCase() == styleName ) return i;
      }
    }
    return -1;
  }

  // ----------------------------------------------------------------------
  function getStyle(styleName) {
    // Find the named style and return it, otherwise return ""
    styleName = styleName.toLowerCase();
    if ( scriptData.hasOwnProperty("Styles") ) {
      for (var i=0;i<scriptData.Styles.length;i++) {
        if ( scriptData.Styles[i][0].toLowerCase() == styleName ) return scriptData.Styles[i][1];
      }
    }
    return "";
  }
  // ----------------------------------------------------------------------
  function addButton(command, buttonList, globalButtons) {
    // Add a local button to the button list, making sure it doesn't
    // conflict with a Global button
      command = command.replace(/^button\s+|^\@\s+/i,"");  // Strip off the "button " or "@ "
      var btnText = "";
      var target = "";
      var temp;

        temp = command.search(",");
        if ( temp != -1 ) {  // We have more than one argument
          // Split it at the first comma
          btnText = command.substring(0,temp);
          btnText = btnText.replace(/^\s+|\s+$/g,""); // Trim leading and trailing whitespace
          target = command.substr(temp+1);            // From first comma to end of string
          target  = target.replace(/^\s+|\s+$/g,"");  // Trim leading and trailing whitespace
        }
        else { // We only have one argument so assume it's the button text and leave target empty
          btnText = command;
        }
//      jscriptLog("addButton: Add local button \"" + btnText + "\", target=\"" + target + "\"");

      // Make sure we don't add a button that conflicts with a global button
      // In version 1.5 of the script engine local buttons now take precedence over global buttons
      // This allows you to define a global button, but override it as needed with a local button
//      i = findButton(btnText, globalButtons);
//      if ( i != -1 )  {
//        // There's already a global button with the same name. Does it have the same target?
//        if ( globalButtons[i][1].toLowerCase() == target.toLowerCase() ) { // It's a duplicate button so just log a message
//          jscriptLog("addButton: A global button labeled \"" + btnText + "\" with target \"" + target + "\" already exists. Local button not added.");
//        }
//        else {  // It's labeled the same but to a different target so this is an error
//          reportScriptError("Script error: Attempt to add local button " + btnText + " but a global button of the same name with target \"" + target + "\"already exists.");
//        }
//      }
//      else {
        buttonList[buttonList.length] = [btnText, target];
//      }
  }
  // ----------------------------------------------------------------------
  function getVariable(str) {
    // This function is used to perform variable substition in expressions and commands.
    // If str is a defined scriptVar this function returns the value of the scriptVar.
    // If str is a numeric string it returns str converted to a number.
    // str can also be one of the following special variables (keywords)
    //    RollOneDice   or Roll1Dice  // Simulates dice rolls with "proper"
    //    RollTwoDice   or Roll2Dice  // odds of each value occuring. See the
    //    RollThreeDice or Roll3Dice  // relevant functions below.
    //    ARG1 to ARGn  Arguments specified when invoking a script label as a command as in
    //      DoSomeEdges 6   // Do 6 edges. See DoSomeEdges.scr
    // Otherwise it just returns str.
    var myName = "getVariable: ";
//    jscriptLog(myName+" Checking for variable named " + str);
    if ( ! isNaN(str) ) {
      // It's a number so just return it in the proper form
      if ( parseInt(str) == parseFloat(str) ) return parseInt(str); // Must be an int
      else return parseFloat(str);  // It's a float. Not a root beer float, a number float.
    }

    // We reach here if str was an alphanumeric string like a variable name, or just a string.
    // Check to see if it's the name of a defined variable
    var temp = scriptVars.get(str);
//    jscriptLog(myName+"temp=" + temp);
    if ( temp != undefined ) {
      // It was a defined scriptvar so return it's value
      if ( ! isNaN(temp) ) {
        // It's a number so just return it in the proper form
        if ( parseInt(temp) == parseFloat(temp) ) return parseInt(temp); // Must be an int
        else return parseFloat(temp);  // It's a float. Not a root beer float, a number float.
      }
      else return temp; // Must be a string so just return it
    }
    else {
      // It was not a defined scriptVar. Was it a special keyword?
      // Check for special keywords or functions
      var lower = str.toLowerCase();
//      jscriptLog(myName+" 1: lower=" + lower);
      if ( lower == "rollonedice"   || lower == "roll1dice" ) return RollOneDice();
      if ( lower == "rolltwodice"   || lower == "roll2dice" ) return RollTwoDice();
      if ( lower == "rollthreedice" || lower == "roll3dice" ) return RollThreeDice();
      if ( lower.search(/^arg[0-9]+$/) != -1 ) {
        temp = parseInt(lower.substr(3),10) - 1 ;  // Get the argument number (1 to n)
        var cmdArgs = parsePageArgs();
        if ( temp < cmdArgs.length ) return cmdArgs[temp];
//        else return "";
      }
      // If it was not one of the special keywords just return the original string
      return str;
    }
  }
  // ----------------------------------------------------------------------
  function makeTimeString(str) {
    // This function takes an input and converts it to a full time
    // specification in the format hh:mm:ss. This was written to support
    // video, since overRide.setVideo() requires the start-at and stop-at arguments
    // to be in strict hh:mm:ss format and does not work with mm:ss for example

    var words = str.split(":"); // Split it on colons
//    jscriptLog("makeTimeString: words = " + words + " (length=" + words.length + ")");
    var timestr = "";
    var hrs = "00";
    var min = "00";
    var sec = "00";
    var temp;
    if ( words.length == 1 ) {
      temp = parseInt(str,10);
      if ( temp < 10 ) sec = "0" + temp; // Add leading zero
      else sec = ""+temp;
    }
    else if ( words.length == 2 ) {
      temp = parseInt(words[0],10);
      if ( temp < 10 ) min = "0" + temp; // Add leading zero
      else min = ""+temp;

      temp = parseInt(words[1],10);
      if ( temp < 10 ) sec = "0" + temp; // Add leading zero
      else sec = ""+temp;
    }
    else if ( words.length == 3 ) {
      hrs = parseInt(words[0],10);

      temp = parseInt(words[1],10);
      if ( temp < 10 ) min = "0" + temp; // Add leading zero
      else min = ""+temp;

      temp = parseInt(words[2],10);
      if ( temp < 10 ) sec = "0" + temp; // Add leading zero
      else sec = ""+temp;
    }
    return hrs + ":" + min + ":" + sec;
  }
  // ----------------------------------------------------------------------
  function processCommands() {
    // Process script commands like GlobalButton and ImageFolder
    var myName = "processCommands: ";
//    jscriptLog(myName+"Starting...");
    var command;

    scriptData = scriptVars.get("scriptData");
    if ( scriptData == "" || scriptData == undefined ) return;
//    jscriptLog(myName+"Script index is " + scriptData.index);

    scriptData.buttons = [];        // Clear any previous local buttons

    var isCommand = true;

    // Process commands from the script until we get a page or something else
    while (isCommand && scriptData.index < scriptData.script.length) {
      isCommand = executeCommand(scriptData.script[scriptData.index]);
      if ( isCommand ) scriptData.index++;  // Process next command
//      jscriptLog(myName+"Script index = " + scriptData.index);
    }
//    jscriptLog(myName+"Finished. Script index = " + scriptData.index);
    scriptVars.put("scriptData", scriptData);
  }

  // ----------------------------------------------------------------------
  function executeCommand(command) {
    // Process the specified script command.
    // The command argument is expected to be a valid script command.
    // If it is a valid script command the return value is true,
    // otherwise the return value is false and the command is a page command or
    // some other unrecognized stuff.

    // Note that "exit" is a common script command, but it is not actually processed by
    // this function. It's flagged as a special command, and then the getNextPage()
    // function is the one that acts on exit commands.

    var myName = "executeCommand: ";
//    jscriptLog(myName+"Starting...");
    var btnText;
    var target;
    var words;
    var temp;
    var command;
    scriptData = scriptVars.get("scriptData");
    if ( scriptData == "" || scriptData == undefined ) return;
    if ( command == "" ) return false;

    var flags;
    var isDelay = /^\d+\.\.\d+$/;   // Regex to test for a delay string like "10..20"
    var isNumeric = /^\d+$/;        // Regex to test for a numeric string

    // Regex for detecting expressions supported by evaluateExpression()
    var isExpression = /^\S+\s*=/;  // Anything beginning with something like "name =" or "name="


    var isCommand = true;

      jscriptLog(myName + command + "");

      // Do a couple quick tests to see if we're done already. If the first word is
      // a double quote, or contains a path or wildcard character ( /, \, or *) we can safely
      // assume it's a file spec, and therefore a page command and just return
//      temp = command.substr(0,command.search(/[\s,{]/));
      temp = command.substr(0,command.search(/[\s,{\(]/));  // Aug 15, 2017 added openeing parenthesis
//      jscriptLog(myName+"First word of command is \"" + temp + "\"");
      if ( temp == "\"" || temp.search(/[\/\*\\]/) != -1 ) {
//        jscriptLog(myName+"Detected a file spec so exiting");
        scriptVars.put("scriptData", scriptData);
        return false;
      }


      // Call replaceTags() before we do anything else
      if ( isSet("dontReplaceTags") ) command = escapeBrackets(command);
      else command = replaceTags(command);
//      jscriptLog(myName+" after replaceTags() command=[" + command + "]");
      if ( command == undefined ) return true;  // replaceTags encountered a fatal error

      // Check the command to see what it is
//      words = command.split(/\s+|,\s*/);  // Split the command using spaces and commas
      words = command.split(/\s+|,\s*|\{|\(|\)/);  // Split the command using spaces and commas and brackets
      //  rightHtml += "Words = " + words + "<br />";

      // Dump the words for debugging
//        for (var i=0;i<words.length;i++) {
//          jscriptLog(myName+"words[" + i + "]=\"" + words[i] + "\"");
//        }

      words[0] = words[0].toLowerCase();  // Force the first word to lower case
//      jscriptLog(myName+"Testing for command=\"" + words[0] + "\"");


      if (command.search(/^\S*:$/) != -1 ) { // This is a label
        // We just skip them because they're boring
//        jscriptLog("myName+" " + command);
      }
      else if (words[0] == "if" ) { // If it's if? Hmmmmm.
        // Extract the expression
//        jscriptLog(myName+"Found an If command: \"" + command + "\"");
        temp = evaluateExpression(command);
//        jscriptLog(myName+"Which evaluates to " + temp);
        if ( temp === false ) scriptData.index++; // Skip the next line if condition is false
      }
      else if ( isExpression.test(command) ) {
//        jscriptLog(myName+"Found an expression: " + command);
        temp = evaluateExpression(command);
//        jscriptLog(myName+"Which evaluates to " + temp);
      }
      else if ( words[0] == "metronome" ) {
        if ( words[1] == "off" ) words[1] = "0";
        scriptData.metronome = parseInt(words[1]);
//        jscriptLog("Set metronome to " + words[1]);
      }
      else if ( words[0] == "timerstyle" ) {
        scriptData.timer = words[1];
//        jscriptLog("Timer style set to " + scriptData.timer);
      }
      else if ( words[0] == "imgfolder" || words[0] == "imagefolder" ) {
        command = command.replace(/^ima*ge*folder\s+/i,"");  // Strip off the "imagefolder " command
        command = command.replace(/^\"|\"\s*$/g,"");  // Strip off quotes
        if ( command != "" ) scriptVars.put("ImageFolder",command);
        else scriptVars.put("ImageFolder","");
//        jscriptLog("ImageFolder:  set to \"" + command + "\" type=" + typeof(command));
      }
      else if ( words[0] == "audiofolder" ) {
        command = command.replace(/^audiofolder\s+/i,"");  // Strip off the command
        command = command.replace(/^\"|\"\s*$/g,"");  // Strip off quotes
        if ( command != "" ) {
          jscriptLog(myName+"Setting AudioFolder to \"" + command + "\"");
          scriptVars.put("AudioFolder",command);
        }
        else {
          jscriptLog(myName+"Deleting variable AudioFolder");
          scriptVars.remove("AudioFolder");
        }

//        jscriptLog("Audio folder set to " + words[1]);
      }
      else if ( words[0] == "videofolder" ) {
        command = command.replace(/^videofolder\s+/i,"");  // Strip off the command
        command = command.replace(/^\"|\"\s*$/g,"");  // Strip off quotes
        jscriptLog(myName+"Setting VideoFolder to \"" + command + "\"");
        if ( command != "" ) scriptVars.put("VideoFolder",command);
        else scriptVars.remove("VideoFolder");

//        jscriptLog("Video folder set to " + words[1]);
      }
      else if ( words[0] == "audio" ) {
        var temp = scriptVars.get("AudioFolder");
        if ( temp != undefined && temp != "" ) scriptData.audio = temp + "/" + words[1];
        else scriptData.audio = words[1];

        scriptData.loop = 0;
        if ( words.length > 2 ) { // Get the loop count parameter
          temp = parseInt(words[2]);
          if ( isNaN(temp) ) {
            jscriptLog(myName+"Invalid loop count on audio command: \"" + command + "\"");
          }
          else
            scriptData.loop = temp;
        }
        jscriptLog("Play audio file " + scriptData.audio + " with loop count " + scriptData.loop);
      }
      else if ( words[0] == "video" ) {
        scriptData.video = parseVideo(command);
      }
      else if ( words[0] == "button" || words[0] == "@") {
        command = command.replace(/^button\s+|^\@\s+/i,"");  // Strip off the "button " or "@ "
        addButton(command, scriptData.buttons, scriptData.globalButtons);
      }
      else if ( words[0] == "globalbutton" || words[0] == "@@" ) {
        command = command.replace(/^globalbutton\s+|^\@\@\s+/i,"");  // Strip off the "globalbutton " or "@@ "
        target = "";
        btnText = "";
        temp = command.search(",");
        if ( temp != -1 ) {  // We have more than one argument
          // Split it at the first comma
          btnText = command.substring(0,temp);
          btnText = btnText.replace(/^\s+|\s+$/g,""); // Trim leading and trailing whitespace
          target = command.substr(temp+1);  // From first comma to end of string
          target  = target.replace(/^\s+|\s+$/g,"");  // Trim leading and trailing whitespace
        }
        else { // We only have one argument so assume it's the button text and leave target empty
          btnText = command;
        }
//        jscriptLog(myName+"Add global button \"" + btnText + "\", target \"" + target + "\"");

        // Make sure we don't add a button that already exists
        i = findButton(btnText, scriptData.globalButtons);
        if ( i != -1 )  {
          // There's already a global button with the same name. Does it have the same target?
          if ( scriptData.globalButtons[i][1].toLowerCase() == target.toLowerCase() ) { // It's a duplicate button so just log a message
            jscriptLog(myName+"A global button labeled \"" + btnText + "\" with target \"" + target + "\" already exists. Button not added.");
          }
          else {  // It's labeled the same but to a different target so this is an error
            reportScriptError("Script error near line " + scriptData.index + ".\nCommand: " + command + "\nA global button labeled \"" + btnText + "\" with target \"" + scriptData.globalButtons[i][1] + "\"already exists.\nButton not added.");
          }
        }
        else {
          // All is well so go ahead and add it.
          scriptData.globalButtons[scriptData.globalButtons.length] = [btnText, target];
        }
      }
      else if ( words[0] == "defaultbutton" ) {
        command = command.replace(/^defaultbutton\s+/i,"");  // Strip off the command
        scriptData.defBtnText = command;
      }
      else if ( words[0] == "removebutton" ) {
        // Syntax is "RemoveButton ButtonText"
        command = command.substr(13);  // Strip off the word "removebutton "
        command = command.toLowerCase();
        command = command.replace(/^\s+|\s+$/g,""); // Trim whitespace

        // Just in case a user copies and pastes a GlobalButton command and we receive something like
        //  RemoveButton Text, target
        // we check for the presence of a comma and ignore everything after it.
        temp = command.search(",");
        if ( temp != -1 ) {
          // Drop everything after the comma
          btnText = command.substring(0,temp);
        }
        else { // We only have one argument so assume it's the button text and leave target empty
          btnText = command;
        }
        btnText = btnText.replace(/^\s+|\s+$/g,""); // Trim leading and trailing whitespace

//        jscriptLog(myName+"Looking for button '" + btnText + "'");
        for (var i=0;i<scriptData.globalButtons.length;i++) {
          if ( scriptData.globalButtons[i][0].toLowerCase() == btnText ) {
            jscriptLog(myName+"removed button " + btnText);
            scriptData.globalButtons.splice(i,1); // Delete element i
            break;
          }
        }
      }
      else if (words[0] == "goto" ) { // Jump to another label
        target = command.substr(5);
        gotoTarget(target);
      }
      else if (words[0] == "gosub" ) { // Jump to a "subroutine" but prep for coming back here
        target = command.substr(6);
        scriptData.index += 1;  // And execute the next command when we return
        goSubTarget(target);  // Jump to the new page or subroutine
        // goSubTarget() either sets the GotoPage flag or updates the script index.
        // If we're going to a page we adjust the script index and end command processing.
        // If we're going to another script label command processing continues there.
        if ( scriptData.hasOwnProperty("GotoPage") && scriptData.GotoPage == true ) {
          isCommand = false;      // To end command processing
        }
      }
      else if (words[0] == "return" ) { // Return from a "subroutine" to the point it was called from
//        if ( scriptData.hasOwnProperty("returnStack") && scriptData.returnStack.length > 0 ) {
//          // We have return points on the stack so do it
//          scriptData.index = scriptData.returnStack.pop(); // Pop the most recent return point from the stack
//          jscriptLog(myName+"Returning to script index=\"" + scriptData.index + "\"");
//        }
//        else {
//          // Hit a return statement but there was no goSub first. See if we got here from a button.
//          // Treat this like an "exit" command to support Call/return or
//          // gosub/return cases, but log a message just in case.
//          jscriptLog(myName+"Found a return command without a gosub. Defaulting to \"exit\"");
//          exitScript();
//          isCommand = false;
//        }
        returnToGosub();

        // If we hit a return without a gosub, we're exiting instead and isSpecialCommand will be set
        if ( scriptData.hasOwnProperty("isSpecialCommand") && scriptData.isSpecialCommand == true ) {
          isCommand = false;  // To terminate command processing
        }
        else {
          // Otherwise, processCommands() assumes if it was a command it needs to increment the
          // script index, but returnToGosub() sets the actual index we need, so we need to subtract
          // one to offset what processCommands is about to do. It's fucking brutal isn't it?
          // Now wonder it takes a special mind to be a programmer.
          scriptData.index = scriptData.index - 1;
        }
      }
      else if (words[0] == "set" ) { // Set one or more flags
        flags = command.substr(4);   // Strips off the word set and leaves us the remaining arguments
        flags.replace(/\s+/g,""); // Remove spaces (so "flag1, flag2" becomes "flag1,flag2")
        comonFunctions.SetFlags(flags, guide.getFlags());
//        jscriptLog(myName+"after flags='" + guide.getFlags() + "'");
      }
      else if (words[0] == "unset" ) { // Unset one or more flags
        flags = command.substr(6); // Strips off the word set and leaves us the remaining arguments
        flags.replace(/\s+/g,""); // Remove spaces (so "flag1, flag2" becomes "flag1,flag2")
        comonFunctions.UnsetFlags(flags, guide.getFlags());
        jscriptLog(myName+"after flags='" + guide.getFlags() + "'");
      }
      else if (words[0] == "ifset" ) { // If the given flag(s) is/are set
          flags = command.substring(6); // Pulls out the flag(s)
          flags.replace(/\s+/g,""); // Remove spaces (so "flag1, flag2" becomes "flag1,flag2")
          if ( ! comonFunctions.canShow(guide.getFlags(), flags, "" ) ) {
            // The required flags are NOT set, so skip the next line of the
            // script, which is the command to execute if they ARE set
            scriptData.index++;
          }
      }
      else if ( words[0].search(/if[not]*set/) != -1  ) { // If the given flag(s) is/are Not set. Feb 12/16 modified to match what's done in loadScript
          flags = command.substring(9); // Pulls out the flag(s)
          flags.replace(/\s+/g,""); // Remove spaces (so "flag1, flag2" becomes "flag1,flag2")
          if ( ! comonFunctions.canShow(guide.getFlags(), "", flags ) ) {
            // The specified flags ARE set, so skip the next line of the
            // script, which is the command to execute if they are NOT set
            scriptData.index++;
          }
      }
      else if ( words[0] == "style" ) {
//        jscriptLog(myName+"Got style command " + command );
        command = command.substr(6);  // Strip off the word "style "
        temp = command.indexOf("=");
        if ( temp != -1 ) {
          words = [];
          words[0] = command.substring(0,temp);  // Split the command using the equal sign
          words[1] = command.substr(temp+1);  // Split the command using the equal sign

          words[0] = words[0].replace(/^\s+|\s+$/g,"");       // Trim leading and trailing whitespace
          words[1] = words[1].replace(/^\s+|\s+$/g,"");       // Trim leading and trailing whitespace
          words[1] = words[1].replace(/^\s*"|\s*"\s*$/g,"");  // Trim quotes

          if ( false ) { // Dump the words for debugging
            for (var i=0;i<words.length;i++) {
              jscriptLog(myName+"words[" + i + "]=\"" + words[i] + "\"");
            }
          }
          // Initialize the Styles property if required
          if ( ! scriptData.hasOwnProperty("Styles") ) scriptData.Styles = [];
          // If the style already exists update it, otherwise add it
          i = findStyle(words[0]);
          if ( i != -1 )  {
            // There's already a style with the same name so update it
            scriptData.Styles[i] = [words[0], words[1]];
//            jscriptLog(myName+"Updated style " + words[0] );
          }
          else {
            // It doesn't exist so add it.
            scriptData.Styles[scriptData.Styles.length] = [words[0], words[1]];
//            jscriptLog(myName+"Added style " + words[0] );
          }
        }
      }
      else if ( words[0] == "edged" ) {
        // Count an edge or edges
        temp = 1; // Default is to add 1
        if ( words.length > 1 ) { // got an argument, presumably a number of edges
          if ( ! isNaN(words[1]) ) {
            temp = parseInt(words[1],10);
          }
        }
        jscriptLog(myName+"Adding " + temp + " edges to count" );
        SECountEdge(temp);
      }
      else if ( words[0] == "exit" ) {
        exitScript();
        isCommand = false;  // To terminate command processing
      }
//      else if ( words[0] == "SomeNewCommand" ) {
//
//      }
      else {
        // Not a recognized command. Check for a page name or script label.
        jscriptLog(myName+"Checking for a page name or script label. Command = " + command);
        // Earlier we forced words[0] to lower case. Re-split the command so we
        // get it in it's original case, since page names are case-sensitive
        words = command.split(/\s+|,\s*/);  // Split the command using spaces and commas

        if ( guide.getChapters().get("default").getPages().containsKey(words[0]) || findScriptLabel(words[0]) != -1 ) {
          // It is a page name or label so treat this like a "gosub target" command
          jscriptLog(myName+"Found a bare pagename or script label: '" + words[0] + "'");

          // Check for arguments to the page
          // Extract the arguments and put them in a scriptVar named pageArgs

          pageArgs = "";
          i = command.indexOf(" ");
          if ( i != -1 ) {
            // Some arguments were found. Chop off the page name and save the arguments
            jscriptLog(myName+"Found page args = \"" + command.substr(i)+ "\"");
            command = command.substr(i);
            command = command.replace(/^\s+|\s+$/g,""); // Trim leading and trailing whitespace
            scriptVars.put("pageArgs", command);
          }
          else {
            jscriptLog(myName+"No page args found in \"" + command + "\"");
            scriptVars.remove("pageArgs");  // Clear previous page args so a page doesn't
                                            // mistakenly get arguments meant for some other page
          }
          scriptData.index += 1;  // So we execute the next command when we return
          goSubTarget(words[0]);  // Set things up to go to the specified page or script index
          // goSubTarget() either sets the GotoPage flag or updates the script index.
          // If we're going to a page we adjust the script index and end command processing.
          // If we're going to another script label command processing continues there.
          if ( scriptData.hasOwnProperty("GotoPage") && scriptData.GotoPage == true ) {
            isCommand = false;      // To end command processing
          }
        }
        else {
          isCommand = false;
        }
      }
//    jscriptLog(myName+"Finished. Was command = " + isCommand + ", script index=" + scriptData.index);
//    scriptVars.put("scriptData", scriptData);
    return isCommand;
  }

  // ----------------------------------------------------------------------
  function parsePageArgs(text) {
    // Parse the arguments supplied to a bare page command.
    // This function properly handles quoted strings.
    // Arguments may be separated by whitespace or commas
    //
    var myName = "parsePageArgs: ";
    if ( text == undefined ) text = scriptVars.get("pageArgs");
    jscriptLog(myName+"Argument list ='" + text + "'");
//    jscriptLog(myName+"typeof(text) = " + typeof(text) );
    if ( typeof(text)== "object" ) text = "" + text; // Force it from a Java String object to a javascript string
    if ( text == undefined || text == "" ) {
//      jscriptLog(myName+"Looks like we got no arguments. Returning " + text);
      return text; // No args so just return
    }
//    jscriptLog(myName+"We got something. Let's see what.");

    var wordStart;
//    var wordEnd;
    var result = [];

    // Originally written to handle arguments passed to pages but also serves as
    // a generic argument list parser.
    // The general syntax of a page reference with arguments is
    //    pagename arguments
    //  The processCommands() function strips the page name and puts the argumments
    //  in a scriptVar named pageArgs.
    // This function retrieves the pageArgs variable and parses it, then returns
    // an array of the results.
    // Call it from the pageload() function of your page.
    //
    // Since processCommands() calls replaceTags() on the command line, all special
    // tags like <RandomFrom>, delays like (min..max) and variable references like
    // <VariableName> will already be replaced, so we dont have to deal with those.

    // Examples
    //  In the examples below delay and metronome are used for illustrative purposes.
    //  Your page can interpret the arguments as whatever is appropriate.
    //
    //  pagename          // No arguments
    //  pagename  delay   // Metronome and text omitted
    //  pagename "this is some text" // Only the page text
    //  pagename delay "this is some text"
    //  pagename  delay,metronome,"this is some text"
    //  pagename  ,metronome, "this is some text"   // Delay omitted, some spaces thrown in
    //  pagename  , metronome, "this is some text"  // Delay omitted, extra spaces thrown in
    //  pagename  ,metronome,"this is some text"    // Delay omitted, no spaces between elements
    //  pagename  delay,,"this is some text"        // Metronome omitted, no spaces between elements

    // Now split it up
    var i = 0;  // String position in argument text
    var n = 0;  // Next argument
    while ( text.substr(i,1) == " " ) i++; // Skip leading whitespace

//    jscriptLog(myName+"Argument string is " + text.length + " characters" );

    while ( i < text.length ) {
//      jscriptLog(myName+"Starting at position " + i + ", character is '" + text.substr(i,1) + "'");
      while ( i < text.length && text.substr(i,1) == " " ) i++; // Skip spaces
      if ( text.substr(i,1) == "\"" ) {
        // Found start of a quoted string
//        jscriptLog(myName+"Found quoted string at " + i );
        i = i + 1;  // Skip the quote
        wordStart = i;
        // Keep going till we find another quote
        while ( i < text.length && text.substr(i,1) != "\""  ) {
          if ( text.substr(i,1) == "\\" ) i = i + 2; // Backslash escapes the next character
          else i = i + 1;
        }
//        jscriptLog(myName+"Quoted string at chars " + wordStart + " to " + i );
        result[n] = text.substring(wordStart,i);
        i = i + 1 // Skip the closing quote
        n = n + 1;
        // Skip possible whitespace and comma following the argument
        while ( i < text.length && text.substr(i,1) == " " ) i = i + 1;
        if ( i < text.length && text.substr(i,1) == "," ) i++; // Skip comma after quoted string
      }
      else if ( text.substr(i,1) == "," ) {  // We got a null argument
//        jscriptLog(myName+"Null arg at char " + i );
        result[n] = "";
        n = n + 1;
        i = i + 1;
      }
      else if ( i < text.length ) {
//        jscriptLog(myName+"Found word starting at " + i );
        wordStart = i;
        // Find end of word
        while ( i < text.length && text.substr(i,1) != " " && text.substr(i,1) != "," ) i = i + 1;
        result[n] = text.substring(wordStart,i);
//        jscriptLog(myName+"Word ends at " + i );
        n = n + 1;
        i = i + 1;
        // Skip possible whitespace and comma following the argument
        while ( i < text.length && text.substr(i,1) == " " ) i = i + 1;
        if ( i < text.length && text.charAt[i] == "," ) i++; // Skip comma after arg
      }
    }
//    jscriptLog(myName+"result.length='" + result.length + "'");
//    if ( true && result.length > 0 ) {  // Dump the args for debugging
//      for ( i=0;i<result.length;i++ ) jscriptLog(myName+"pageArg[" + i + "]=" + result[i]);
//    }
    return result;
  }
  // ----------------------------------------------------------------------
  function evaluateExpression(expression) {
    var myName = "evaluateExpression: ";
//    jscriptLog(myName+"Parsing \"" + expression + "\"");
    var result = "";
    // The expression is parsed and sanitized, all variable
    // references are replaced with their current values, the expression is
    // evaluated and the result is returned.
    //
    // If the expression contains an error the result is the javascript constant
    // "undefined" so you can use code like
    //   if ( evaluateExpression(someExpression) === undefined ) // then the expression failed
    // Otherwise for an "If" command the expression is evaluated and the return
    // value is true or false.
    // For an assignment type expression like
    //   myVariable = someOtherVariable * someNumber
    // the expression is evaluated and the result assigned to myVariable, and
    // the result is also returned to the caller.

    // Sample expressions
    //  a = 100
    //  b = a * 10
    //  b = a * 10 - 60
    //  if ( a == b )
    //  if ( a < b )
    //  a <= b
    //  if isSet(a)
    //  if ( isSet(a) )
    //  isNotSet(a)
    //  isNotSet(a+b)
    //  isNotSet(a|b)
    //  isSet(a) and isNotSet(b)
    //  b > 5 and isNotSet(a)
    //  a < 5 and (isSet(a) and isNotset(b))
    //
    // What we need to do:
    //  1. Find arguments like a and b, check to see if they are defined variables
    //     and replace them with their current value
    //  2. Find special functions like isSet, isNotSet etc. and wrap their arguments in quotes so
    //     isSet(a) becomes isSet("a") and isSet(a+b) becomes isSet("a+b")
    //  3. Pass everything else through unchanged
    var start = 0;
    var isFunctionArg = false;
    var target = "";

    //  Determine the type of expression we're dealing with.
    //  It could be an assignment or an if statement
    var isIfCommand = true;  // We assume it's an if conditional
    // Now test it to see if we have something like "name = something" which is
    // an assignment rather than an if conditional
    if (expression.search(/^\s*\S+\s*=\s*/) != -1 ) {
      isIfCommand = false; // It's an assignment type expression
    }

    if ( isIfCommand ) {
      // Chop the word "if" from the command
      start = expression.search(/^\s*if[\s(]*/i);
      if ( start != -1 ) { // found the word "if" at the beginning of the line so remove it
        expression = expression.substr(start+2);
      }
      start = 0;  // Start at the beginning of the string
//      jscriptLog(myName+"Stripping \"if\" yields \"" + expression + "\"");
    }
    else {
      expression = expression.replace(/\s*=\s*/,"="); // Strip spaces around first equal sign
//      jscriptLog(myName+"Stripping spaces around equals sign yields \"" + expression + "\"");
      start = expression.indexOf("=");  // Start at the equal sign so we skip over the target variable name
      target = expression.substr(0,start);
      start = start + 1;  // Skip the equal sign
      // Don't allow assignment to special keywords
      if ( (target.search(/^arg[0-9]+$/i) != -1) || (target.search(/^roll.*dice$/i) != -1) ) {
        reportScriptError("**Error** Assignment to special keyword " + target + " is not allowed.");
        return undefined;
      }
    }
    while ( start < expression.length ) {
      // Find the next word in the expression
      // Copy whitespace, brackets and operators
      while (start < expression.length && expression.charAt(start).search(/[\s\(\)\<\>\=\!\+\-\*\/]/) != -1 ) {
        result += expression.charAt(start); // Copy whitespace, brackets and operators
        start++;
      }

//      jscriptLog(myName+"start=" + start + ", char=\"" + expression.charAt(start) + "\"");
      if ( start >= expression.length ) break;  // We're done
      var end = start + 1;

      if ( expression.charAt(start) == "\"" ) {
        // Found start of a quoted string
        // At this point we have start = end and the "word" we got was the quote
        // Keep going till we find another quote
        while ( end < expression.length && expression.charAt(end) != "\""  ) {
          if ( expression.charAt(end) == "\\" ) end = end + 2; // Backslash escapes the next character
          else end = end + 1;
        }
        if ( end < expression.length ) end = end + 1 // To include the closing quote
        else {
          jscriptLog(myName+"Warning: String is missing closing quote in " + expression );
          expression = expression + "\""; // Add closing quote
          end = expression.length // Include the closing quote
        }
        jscriptLog(myName+"Found quoted string at chars " + start + " to " + end );
        jscriptLog(myName+"Resulting string is " + expression.substring(start,end) );
        // Handle possible formatted text in the quoted string
        result += handleTextFormats(expression.substring(start,end));
        jscriptLog(myName+"After handleTextFormats string is " + expression.substring(start,end) );
        isFunctionArg = false; // If the string was a function arg we're finished with that now
                               // This may fail if the function has multiple arguments - Need to test that
      }
      else {
        // We have the start of some other expression element
        // Find the next delimiter character (opening or closing parentheses, operator or space)
        // If we are parsing a special function argument we ignore plus signs
        if ( isFunctionArg ) while (end < expression.length && expression.charAt(end).search(/[\s\(\)\<\>\=\!\-\*\/\"]/) == -1 ) end++;
        else while (end < expression.length && expression.charAt(end).search(/[\s\(\)\<\>\=\!\+\-\*\/\"]/) == -1 ) end++;
//        jscriptLog(myName+"end=" + end + ", char=\"" + expression.charAt(end) + "\"");

        var word = expression.substring(start,end);
//        jscriptLog(myName+"next word=\"" + word +"\"" );

        if ( isFunctionArg ) {
          // This word is the argument to a special fuunction so don't alter it
          // other than enclosing it in quotes
//          jscriptLog(myName+"Adding function arg=\"" + word +"\" to \"" + result + "\"" );
          result += word + "\"";  // And end the function argument with a closing quote
          isFunctionArg = false;  // Because we're finished that one now
        }
        else {
          // Check the current word to see if it's a variable or special function name

          var value = getVariable(word);
//          jscriptLog(myName+"value = " + value);
          if ( value != word ) {
            // It was a defined variable or keyword so insert it's value
//            jscriptLog(myName+"Replacing variable name with value = " + value);
            if ( isNaN(value) ) result += "\"" + value + "\"";  // It's a string
            else result += value; // It's numeric
          }
          else {
            // Is it a special function or operator name?
//            jscriptLog(myName+"Check for special functions and operators in \"" + word + "\"");

            // Test against the lower case forms of isSet and isNotSet and insert the
            // proper spelling in case the user gets sloppy and types isset instead of isSet
            if ( word.toLowerCase() == "isset" ) {
              result += "isSet(\"";
              while (end < expression.length && expression.charAt(end).search(/[\(\s]/) != -1 ) end++;  // Skip whitespace and opening bracket
              isFunctionArg = true;
            }
            else if ( word.toLowerCase() == "isnotset" || word.toLowerCase() == "notset") {
              result += "notSet(\"";  // Also adds the opening quote to change isNotSet(a) to isNotSet("a")
              while (end < expression.length && expression.charAt(end).search(/[\(\s]/) != -1 ) end++;  // Skip whitespace and opening bracket
              isFunctionArg = true;
            }
            else if ( word.toLowerCase() == "isdefined" ) {
              while (end < expression.length && expression.charAt(end).search(/[\(\s]/) != -1 ) end++;  // Skip whitespace and opening bracket
              if ( expression.charAt(end) != "\"" ) result += "isDefined(\"";  // Also adds the opening quote to change isDefined(a) to isDefined("a")
              else result += "isDefined(";  // It appears the argument is already enclosed in quotes
              isFunctionArg = true;
            }
            else if ( word.toLowerCase() == "notdefined" ) {
              while (end < expression.length && expression.charAt(end).search(/[\(\s]/) != -1 ) end++;  // Skip whitespace and opening bracket
              if ( expression.charAt(end) != "\"" ) result += "! isDefined(\"";  // Also adds the opening quote to change isDefined(a) to isDefined("a")
              else result += "! isDefined(";  // It appears the argument is already enclosed in quotes
              isFunctionArg = true;
            }
            else if ( word.toLowerCase() == "and" ) {
              result += "&&"; // Change the word "and" to the proper operator
            }
            else if ( word.toLowerCase() == "or" ) {
              result += "||"; // Change the word "or" to the proper operator
            }
            else {
              // Whatever it was just tack it on
              if ( isNaN(word) ) jscriptLog(myName+"Unrecognized word or expression element, or possible undefined variable: \"" + word + "\"");
              result += word;
            }
          }
        }
      }
      start = end;  // Start here on next loop
//      jscriptLog(myName+"interim result = \"" + result +"\"");
    }
    if ( isIfCommand ) {
//        jscriptLog(myName+"Final condition = \"" + result +"\"");
            // In an if command a common mistake is to write "if ( a = b )" instead
            // of "if ( a == b )" We check for a single equal sign and correct that
            // if required and support lazy coding and the ignorance of the masses.
            // Mwah ha ha!
            // In this command
            result = result.replace(/[^\!\<\>\=]=[^=]/,"==");
            // The pattern /[^!<>=]=[^=]/ translates to
            //  A character that is not one of !, <, >, or =
            //  Followed by an equal sign
            //  Followed by another character that is not an equal sign
            // We could have anything like
            //    a=b   (incorrect)
            //    a==b  (correct syntax)
            //    a <= b
            //    a != b
            //    a >= b
            //    a!=b  (no extra spaces
            //
            result = result.replace(/[^!<>=]=[^=]/,"==");
//        jscriptLog(myName+"After checking for single equal signs condition = \"" + result +"\"");
        try {
              result = eval(result);
        }
        catch(err) {
            reportScriptError("Evaluation of expression \n&nbsp;&nbsp;if " + result + "\n failed with error \n&nbsp;&nbsp;" + err.message);
            result = undefined;
        }
        jscriptLog(myName+"Which evaluates to \"" + result +"\"");
    }
    else {
//        jscriptLog(myName+"Final expression is \"" + target + " = " + result +"\"");
        try {
              result = eval(result);
        }
        catch(err) {
            reportScriptError("Evaluation of expression \n&nbsp;&nbsp;" + target + " = " + result + "\n failed with error \n&nbsp;&nbsp;" + err.message);
            result = undefined;
        }

        if ( result === "" ) {
          scriptVars.remove(target);
//          jscriptLog(myName+"scriptVar " + target + " was removed");
        }
       else {
          scriptVars.put(target,result);
          jscriptLog(myName+"scriptVar " + target + " set to \"" + result + "\"");
        }
    }
    return result;
  }
  // ----------------------------------------------------------------------
  function parseVideo(command) {
      jscriptLog("parseVideo: Starting with '" + command + "'");
      // Parse a video command and return an array of the results
      // We could have any of the following cases
      //    video myVideo.mp4
      //    video myVideo.mp4 next
      //    video myVideo.mp4 start-at
      //    video myVideo.mp4 start-at stop-at next
      //    video myVideo.mp4,,stop-at next
      // The final argumet can be "wait" or "next"
      // Wait waits for the user to click a button, next goes to the next page.

      var video = [];
      // set defaults
      video.filename = "";
      video.startAt = ""; // Start at the beginning
      video.stopAt = "";  // Play till the end
      video.atEnd = "wait";   // Then wait for a button press

      scriptVars.put("pageArgs",command);
      var words = parsePageArgs();

      // We know words[0] is "video" or we wouldn't be here

      // Grab the video name/path
      var temp = scriptVars.get("VideoFolder");
      jscriptLog("parseVideo: VideoFolder is " + temp);
      if ( temp != undefined && temp != "" ) video.filename = temp + "/" + words[1];
      else video.filename = words[1];

      // Now see what other arguments were given
      if ( words.length > 2 ) {
        words[2] = words[2].toLowerCase();
        if ( words[2] == "next" || words[2] == "wait" ) { // No start-at given but got a wait/next
          video.atEnd = words[2];
        }
        else {  // Assume we got a start-at argument
          video.startAt = makeTimeString(words[2]);
        }
      }

      if ( words.length > 3 ) {
        words[3] = words[3].toLowerCase();
        if ( words[3] == "next" || words[3] == "wait" ) { // No end-at given but got a wait/next
          video.atEnd = words[3];
        }
        else {  // We got a stop-at argument
          video.stopAt = makeTimeString(words[3]);
        }
      }
      if ( words.length > 4 && words[4].toLowerCase() == "next" ) video.atEnd = "next";

      // In the overRide.setVideo() command the atEnd parameter actually specifies
      // a target page. Here we convert the atEnd parameter into an appropriate
      // page name. To wait, we give a null name, and for next we use runScript
      // to run the next command.
      if (video.atEnd == "wait") video.atEnd = "";
      else video.atEnd = "runScript";

      jscriptLog("parseVideo: Filename=\"" + video.filename + "\" startAt='" + video.startAt + "' stopAt='" + video.stopAt + "' atEnd='" + video.atEnd + "'");
      return video;
  }
  // ----------------------------------------------------------------------
  function getNextPage() {
    var myName = "getNextPage: ";
    jscriptLog(myName+"Starting...");
    // This function parses the page descriptor and returns an object with
    // the page details.
    // A page descriptor has the following form:
    //    image name, delay, metronome, page text
    // All arguments except image name are optional
    //
    // Call this function with a statement like
    //    nextPage = getNextPage();
    // nextPage will be an object with at least the following fields, and usually
    // many more that you won't care about.
    //
    //    image     The image to display
    //    text      The page text
    //    delay     The delay to be used, or 0 for no delay
    //    metronome The metronome value, or 0 for no metronome
    //

    var i;
    var result = "";
    var temp;
    var temp2;
    var delim;
    var delim2;
    var finished = false;
    var isRandom = /^\S+\.\.\S+$/;   // Regex to test for a delay string like "10..20"
    var command;

//    // Define some regular expressions for testing the arguments
//    var isNumeric = /^\d+$/;        // Tests for a numeric string

    var scriptData = scriptVars.get("scriptData");

    // Check for basic error conditions
    if ( scriptData == "" || scriptData == undefined ) {
      // No script loaded
      scriptData = {};  // Create a simple page to return the error notice in
      scriptData.image = "";
      scriptData.text = "No script has been loaded.\nYou must use the loadScript()function to load a script file before transfering control to the runScript page.";
      scriptData.delay = 0;
      scriptData.metronome = 0;
      scriptData.audio = "";
      scriptData.video = [];
      scriptData.buttons = [];
      scriptData.globalButtons = [];
      scriptVars.put("scriptData", scriptData);  // Save all that stuff
      return scriptData;  // And go do it man. Or woman. Or boy. Or, er, whatever.
    }

    // We have a script loaded. Check for special situations.
    // This check catches special commands triggered by buttons.
    if ( scriptData.hasOwnProperty("GotoPage") && scriptData.GotoPage == true ) return scriptData; // We are being redirected to a Guideme page
    if ( scriptData.hasOwnProperty("isSpecialCommand") && scriptData.isSpecialCommand == true ) return scriptData;

    // Nothing special going on. Let's see if we can find a page command.
    // Set defaults for the page
    var delay = 0;          // Default is no delay
    scriptData.audio = "";  // And no audio
    scriptData.video = [];  // And no video
    scriptData.text = "";   // Erase any previous text
    scriptData.GotoPage = false; // Used for handling gotoPage commands
    scriptVars.put("scriptData", scriptData);  // Save all that stuff

    processCommands(); // Handle script commands like GlobalButton and Metronome

    scriptData = scriptVars.get("scriptData");

    // And now check for special commands again.
    // This check catches special commands actually coded in the script.
    if ( scriptData.hasOwnProperty("GotoPage") && scriptData.GotoPage == true ) return scriptData;
    if ( scriptData.hasOwnProperty("isSpecialCommand") && scriptData.isSpecialCommand == true ) return scriptData;
//    jscriptLog(myName+"script index=\"" + scriptData.index + "\"");
//    jscriptLog(myName+"command= " + scriptData.script[scriptData.index]);

    // The next script command (if any) is now guaranteed to be either a page
    // descriptor, or some messed up error situation like prematurely hitting
    // the end of the script

    if ( scriptData.index >= scriptData.script.length ) {
      // Reached end of script file so exit
      scriptData.text = "exit";
      scriptData.audio = "";
      scriptData.delay = 0;
      scriptData.metronome = 0;
//      scriptData.isSpecialCommand = true;
      scriptVars.put("scriptData", scriptData);  // Save all that stuff
      return scriptData;  // And go do it...
    }

    // Get the page command
    var command = scriptData.script[scriptData.index];
    command.replace(/^\s+|\s+$/g,""); // Trim leading and trailing whitespace
    jscriptLog(myName+"command=\"" + command + "\"");

    // Replace special tags before processing to allow special
    // tags like (min..max), <variable>, <RandomFrom..> etc.
    if ( isSet("dontReplaceTags") ) command = escapeBrackets(command);  // For error reporting
    else command = replaceTags(command);
    if ( command == undefined ) return;  // replaceTags or escapeBrackets encountered a fatal error

//    jscriptLog(myName+"after replacing tags command=\"" + command + "\"");

        // *********************************************************************
        // We should finally have a page descriptor and we can get to work
        // *********************************************************************

        // Okay, so far so good. No major errors and it wasn't a gotoPage command
        // so we assume it's a page description.
        // Let's see what arguments we have.
        // Everything is optional so we jump through some hoops to identify what
        // arguments have been specified and what have been left off.
        //
        // Possibilities are numerous but include:
        // image
        // image, Page text
        // image { Page text }
        // image, delay, Page text
        // image, delay, metronome, Page text
        // delay Page text
        // Page text  //just new text, reusing the same image with no delay or metronome
        // Page text may be a simple string or a block of text like
        // {
        //    multi-line text block
        // }

        // ******************************************************************
        // First we pull off the image name
        // ******************************************************************

          //  Parse the first word using comma, space or brace
          result = "";

          delim = command.search(/[\s,{]/);  // Find first delimiter
          if ( delim == -1 ) { // No delimiter found
            temp = command;     // All we got is a single word with no spaces or anything
            delim = command.length;
          }
          else {
            // Try everything up to the first delimiter
            temp = command.substr(0,delim);     // Extract the potential image name
//            jscriptLog(myName+"Found a \"" + command.charAt(delim) + "\" at position = " + delim + " so trying " + temp);
          }

          if ( temp == "\"" ) { // If it's a double quote just reuse the previous image
            result = temp;
          }
          else {
            // Is what we have a valid image spec?
            result = getImage(temp);
//            jscriptLog(myName+"verifyImage 1 returned \"" + result + "\"");
            if ( result == "" ) { // That didn't match anything.
              // Did we maybe have a wildcard spec that didn't match any files, or just a non-existent image
              if ( temp.search(/\*/) != -1 || temp.search(/^.*\.[jpg]+[pni]+[egf]+[g]*$/i) != -1 ) {
                // We either got a wildcard spec that didn't match anything or an
                // image spec like somefile.<jpg,png,gif> but the file doesn't exist
                jscriptLog(myName+"1: Image " + temp + " - no match or not found.");
              }
              else {
                // It's not one of those so try again using the first comma or brace as
                // the delimiter in case we had a filename with spaces in it.
                delim2 = command.search(/[,{]/);  // Find first comma or brace
//                jscriptLog(myName+"Looking for comma or brace we got " + delim2 );
                if ( delim2 == -1 ) {
                  delim = -1;  // Looks like there is no image spec
                }
                else {
                  // We found a comma or brace
                  temp2 = command.substr(0,delim2);  // Extract the potential image name
                  // Trim trailing whitespace
                  temp2 = temp2.replace(/\s+$/g,""); // Trim trailing whitespace
//                  jscriptLog(myName+"Found a \"" + command.charAt(delim2) + "\" at position = " + delim2 + " so trying " + temp2);
                  result = getImage(temp2);
                  if ( result == "" ) {
                    // Was this one a wildcard spec that didn't match any files, or just a non-existent image
                    if ( temp.search(/\*/) != -1 || temp.search(/^.*\.[jpg]+[pni]+[egf]+[g]*$/i) != -1 ) {
                      // We either got a wildcard spec that didn't match anything or an
                      // image spec like somefile.<jpg,png,gif> but the file doesn't exist
                      jscriptLog(myName+"2: Image " + temp + " - no match or not found.");
                      delim = delim2;  // Ignore this image portion
                    }
                  }
                  else {
                    // That's better. Now we have something.
                    delim = delim2;
                  }
                }
              }
            }
          }
//          jscriptLog(myName+"Delim = " + delim );
          // Now, did we get a valid result?

//          jscriptLog(myName+"Final image spec is \"" + result + "\"");
          if ( result == "" ) {
            // It's possible we have a page command with an image spec, but the
            // image name is spelled wrong or doesn't exist.
            // Just log a message and re-use the previous image
            jscriptLog(myName+"No valid image spec found in \"" + command + "\". Reusing previous image.");
          }

          // Chop the image spec from the command
          if ( command.charAt(delim) == "{" ) command = command.substr(delim); // So we keep the {
          else command = command.substr(delim+1); // To chop the delimiter as well
//          jscriptLog(myName+"After chopping the image spec command = \"" + command + "\"");

          if ( result != "" && result != undefined && result != "\"" ) {
            // We got a new image spec so update it
            scriptData.image = result;
          }
//          jscriptLog(myName+"imageSpec=" + scriptData.image);

        // *********************************************************************
        // Now we are left with a string consisting of zero or more arguments
        // to the page command.
        // These will be delay, metronome and page text
        // *********************************************************************

        // *********************************************************************
        // Check for a Delay argument
        // *********************************************************************
        if ( notSet("KeepDelays") ) scriptData.delay = 0; // We don't keep it by default but this lets us turn that on.
        temp = "";
        if ( command != "" ) {
          // See if we have a delay argument.
          // If we had something like
          //  image, ,metronome,text
          //  image,, metronome,text
          //  image, delay,, text
          // to omit one of the arguments, at this point the image part has been stripped and we'd have
          //   ,metronome,text
          //   delay,, text
          // or possibly
          // delay metronome text
          command = command.replace(/^\s+/,""); // Trim leading whitespace
//          jscriptLog(myName+"Checking for Delay in command='" + command + "'");
          i = command.search(/[\s+,{]/); // Finds a space, comma or opening brace
          if ( i == -1 ) {  // No delimiter found so this is either a delay and everything else is omitted, or it's the page text.
            temp = command;
            i = command.length;
          }
          else {  // We have a delimiter
            temp = command.substr(0,i);     // Extract the possible delay part
            if ( temp == "" ) { // The delay was omitted in a command like image,,metro Or image {text}
              if ( command.substr(i,1) == "{" ) command = command.substr(i);
              else command = command.substr(i+1);
            }
          }

          if ( temp != "" ) {
//              jscriptLog(myName+"Found possible delay='" + temp + "'");
             if ( isRandom.test(temp) ) {
//               jscriptLog(myName+"Found an n..m form delay");
               // It's a string like "10..20" or "min..max"
               temp = temp.split("..");
//               jscriptLog(myName+"min=" + temp[0] + ", max=" + temp[1]);
               scriptData.delay = RandomFrom(getVariable(temp[0]),getVariable(temp[1]));
               if ( command.substr(i,1) == "{" ) command = command.substr(i);
               else command = command.substr(i+1);
             }
             else {
//              jscriptLog(myName+"Not an n..m form delay");
             // If the string is not numeric see if it's a variable
               if ( isNaN(temp) ) temp = getVariable(temp);
               if ( ! isNaN(temp) ) {
                // Looks like we got a delay specification. Force it to be an integer
                scriptData.delay = parseInt(temp);
                // Chop the argument from the command
                if ( command.substr(i,1) == "{" ) command = command.substr(i);
                else command = command.substr(i+1);
              }
            }
          }
        }

        // *********************************************************************
        // Check for a Metronome argument
        // *********************************************************************
        // After stripping image and delay we could have
        //   metronome,text
        //   , text
        //   text
        //   { text ....
        if ( notSet("KeepMetronomes") ) scriptData.metronome = 0; // We don't keep it by default but this lets us turn that on.
        temp = "";
        if ( command != "" ) {
         // Do we have a metronome argument?
          command = command.replace(/^\s+/,""); // Trim leading whitespace
//          jscriptLog(myName+"Checking for metronome in command='" + command + "'");
          i = command.search(/[\s+,{]/); // Finds a space, comma or opening brace
          if ( i == -1 ) {  // No delimiter found so test the whole string
            temp = command;
            i = command.length;
          }
          else {  // We have a delimiter
            temp = command.substr(0,i);     // Extract the possible metronome part
          }

          if ( temp != "" ) {
//             if ( isRandom.test(temp) ) {
//               jscriptLog(myName+"Found a metronome in n..m form");
//               // It's a string like "10..20" or "min..max"
//               temp = temp.split("..");
//               scriptData.metronome = RandomFrom(getVariable(temp[0]),getVariable(temp[1]));
//               jscriptLog(myName+"min=" + temp[0] + ", max=" + temp[1] + ", result=" + scriptData.metronome);
//               if ( command.substr(i,1) == "{" ) command = command.substr(i);
//               else command = command.substr(i+1);
//             }
//             else {
//              jscriptLog(myName+"Not an n..m form metronome");
              // If the string is not numeric see if it's a variable
              if ( isNaN(temp) ) temp = getVariable(temp);
              if ( ! isNaN(temp) ) {
                // Looks like we got a variable for a metronome specification. Force it to be an integer
                scriptData.metronome = parseInt(temp);
                // Chop the argument from the command
                if ( command.substr(i,1) == "{" ) command = command.substr(i); // Don't chop the opening brace
                else command = command.substr(i+1); // But chop any other delimiter
              }
//            }
          }
        }

        // *********************************************************************
        // And anything left must be page text
        // *********************************************************************

        command = command.replace(/^\s+/,""); // Trim leading whitespace
//        jscriptLog(myName+"Looking for text in '" + command + "'");
        if ( command == "" ) {  // No text was given
          text = "<br/>";  // No text found. What's up with that? Nothing to say?
//          jscriptLog(myName+"No page text found.");
        }
        else { // We have some text
          // At this point we have these possibilities
          // 1) What's left in command is the entire page text, possibly like { page text }
          // 2) Command is the opening brace indicating the start of a block of text
          // 3) Or we have the beginning of a block that's continued on subsequent lines like
          //    { first line of text
          //      second line of text
          //      etc.

          // Check for an opening brace that starts a text block
          i = command.search(/^\s*{/); // Matches "{" as the first non-whitespace character
          if ( i == -1 ) {
            // It's all text
            text = command;
            text = handleTextFormats(text);   // Handle format codes in text
//            jscriptLog(myName+"Found text '" + text + "'");
          }
          else {
//            jscriptLog(myName+"Start of text block found.");
            // We have the start of a text block
            // See if we have the complete text block like { page text here }
            command = command.substr(i+1); // Everything from the brace on is page text
            if ( command.charAt(command.length-1) == "}" ) { // we have the whole block
              text = command.substr(0,command.length-1);  // Chop off the closing brace
//              jscriptLog(myName+"Found text '" + text + "'");
              text = handleTextFormats(text);   // Handle format codes in text
            }
            else {
              // We had the beginning of a multi-line text block
//              jscriptLog(myName+"Scanning text block...");
//              jscriptLog(myName+"Starting with text='" + command + "'");

              command = handleTextFormats(command);   // Handle format codes in this chunk
              text = command; // And this is the beginning of our text

              // And now we append the following lines until we find the closing brace
              scriptData.index += 1;
              var finished = false;
              while ( ! finished && scriptData.index < scriptData.script.length ) {
                // We keep eating script lines until we find the end of the text
                // block or the end of the script, whichever comes first
                command = scriptData.script[scriptData.index];
//                jscriptLog(myName+"Next line of text block is '" + command + "'");

                // Handle tag replacements in the text block
                if ( isSet("dontReplaceTags") ) command = escapeBrackets(command);
                else command = replaceTags(command);
                if ( command == undefined ) return;  // replaceTags or escapeBrackets encountered a fatal error

//                jscriptLog(myName+" After replacing tags it's '" + command + "'");
                if ( command == "}" ) {
                  finished = true;
                  command = "";
                }
                else if ( command.substr(command.length-1) == "}" ) {
                  command = command.substr(0,command.length-1); // Chop off closing brace
                  finished = true;
                }
//              else {
//                It's all page text and we don't have to do anything
//              }

                // Now just to complicate things we may have certain commands
                // embedded in the text block

//                jscriptLog(myName+"Checking for embedded commands '" + command + "'");
                if ( command.search(/^\s*button\s+|^\s*\@\s+/i ) != -1 ) { // Check for embedded button command
//                   jscriptLog(myName+"Found a button command");
//                  command = command.replace(/^\s*button\s+|^\s*\@\s+/i,"");  // Strip off the "button " or "@ "
                  addButton(command, scriptData.buttons, scriptData.globalButtons);
                  command = ""; // So it doesn't get added to the page
                }
                else if ( command.search(/^\s*audio\s+/i ) != -1 ) { // Check for embedded audio command
//                  jscriptLog(myName+"Found an embedded audio command");
                  // Strip off the "audio " command
                  command = command.replace(/^\s*audio\s+/i,"");
                  // And whatever is left is assumed to be the arguments
                  var args = command.split(/\s+|,\s*/);
                  temp = scriptVars.get("AudioFolder");
                  if ( temp != undefined && temp != "" ) scriptData.audio = temp + "/" + args[0];
                  else scriptData.audio = args[0];
                  scriptData.loop = 0;
                  if ( args.length > 1 ) {
                    temp = parseInt(args[1]);
//                    jscriptLog(myName+"loop arg is " + args[1] + " and temp=" + temp);
                    if ( isNaN(temp) ) {
                      jscriptLog(myName+"Invalid loop count on audio command: \"audio " + command + "\"");
                      scriptData.loop = 0;
                    }
                    else
                      scriptData.loop = temp;
                  }
                  command = ""; // So it doesn't get added to the page

                }
                else if ( command.search(/^\s*video\s+/i ) != -1 ) {
//                  jscriptLog(myName+"Found a video command");
                  scriptData.video = parseVideo(command);
                  command = ""; // So it doesn't get added to the page
                }

                if ( command != "" ) {
                  command = handleTextFormats(command);   // Handle format codes in the text
                  // And append the line to the text
//                  jscriptLog(myName+"Adding text '" + command + "'");
//                  jscriptLog(myName+"Current flags are " + guide.getFlags() );

                  if ( notSet("AutoLineBreaks+AutoLineBreak") ) {
                    // Use the original method of concatenating all lines in the text block
                    text = text + " " + command;
                  }
                  else {
                    // Automatically add line breaks to the lines of text in the text block
                    // unless the previous line ends in an underscore (_).

//                  jscriptLog(myName+"last char of text is  " +  text.substr(text.length-1,1));
//                  jscriptLog(myName+"last two chars are    " +  text.substr(text.length-2,2));
//                  if ( text.search(/\<br\s*\/\>$/i) != -1 ) jscriptLog(myName+"Text ends in <br/> so not adding line break");

                    if ( text.substr(text.length-1,1) == "_" ) {
                      // Previous line ended in a continuation char
                      text = text.substr(0,text.length-1); // Chop off the continuation char
                      text = text + " " + command;  // Append the current line
                    }
                    else if ( text.substr(text.length-2,2) != "\\n" && text.search(/\<br\s*\/\>$/i) == -1 ) {
                      // If the current text does not already end in a new line or <br/> add one
                      text = text + "<br/>" + command;
                    }
                    else {
                      // Otherwise just add the line
                      text = text + command;
                    }
                  }
                }
                if ( ! finished ) scriptData.index += 1;
              }  // End while
            } // End read of text block
          }  // End text block processing
        } // End of text argument processing
//        jscriptLog(myName+"Final page text is '" + text + "'");

        // Well, we're finally done parsing that nonsense.
        if ( text == "" ) scriptData.text = "<br/>";
        else scriptData.text = text;

          // Do we have more commands?
        if ( scriptData.index < scriptData.script.length ) {
          scriptData.index = scriptData.index + 1;  // Next command to execute
//          overRide.addButton("runScript","Step","","","",""); // FOR DEBUGGING ONLY
        }

        // And return the description for this page
        // All of the page elements in scriptData should have been set above

        if ( false ) {
          jscriptLog(myName+"Image= " + scriptData.image);
          jscriptLog(myName+"delay= " + scriptData.delay);
          jscriptLog(myName+"Metro= " + scriptData.metronome);
          jscriptLog(myName+"Text = " + scriptData.text);
          jscriptLog(myName+"audio= " + scriptData.audio);
          jscriptLog(myName+"video= " + scriptData.video);
        }
        scriptVars.put("scriptData", scriptData);  // Save it for next iteration

//      jscriptLog(myName+"Finished.");
      return scriptData;
    }


  // ----------------------------------------------------------------------
  function getImage(imageSpec) {
    var myName = "getImage: ";
//    jscriptLog(myName+"Testing \"" + imageSpec + "\"");
    // Take the supplied imageSpec and see if it resolves to a valid image
    // The return value is the imageSpec if valid, or null otherwise
    var result;
    var temp = imageSpec;
    var i;
      // Do we just have with a double quote telling us to keep the previous image?

      result = "";
      if ( temp == "\"" ) { // If we get a double quote
        result = temp;        // Just keep it
      }
      else {  // We got something else so see if it's a valid image spec
        var imgFolder = scriptVars.get("ImageFolder");
        if ( ! imgFolder || imgFolder === undefined ) imgFolder = "";
//        jscriptLog(myName+"ImageFolder is \"" + imgFolder + "\"");

        // If the first character is a not slash add the image folder
        if ( temp.substr(0,1) != "/" ) {
            temp = imgFolder + "/" + temp;
        }
        result = matchFile(temp);
      }
      jscriptLog(myName+"Final result is \"" + result + "\", type=" + typeof(result) );
      return result;
  }

  // ----------------------------------------------------------------------
  function matchFile(file) {
    // Take the supplied file spec and see if it resolves to a valid filename
    // allowing for wildcards (*)
    // The return value is the full file spec if valid, or null otherwise
    var myName = "matchFile: ";
    var result;
    var temp = file;
//    jscriptLog(myName+"Testing file \"" + temp + "\" type = " + typeof(temp));
    var i;

    // Check for wildcards like *, *.jpg, Session*.scr
    if (temp.search(/\*/) != -1 ) {
      // Found a wildcard in the file spec
//      jscriptLog(myName+"Got wildcard file spec.");

      // Through version 1.5 GuideMe has a "bug" where GetRandomFile will return
      // folders as well as files, which of course doesn't work well if you're
      // trying to match a file.
      // On Windows systems it will also sometimes return the hidden system file
      // thumbs.db.
      // To compensate for this we try up to 10 times to find a file, checking
      // each result to see if it's a folder, or the dreaded thumbs.db.

      var tries = 0;
      var done = false;
      while ( ! done ) {
        // comonFunctions.GetRandomFile() requires us to specify the path and
        // image portions separately so we have to sort that out.

        if ( temp.search(/\//) != -1 ) {
          // The file spec contains a path portion
          i = temp.lastIndexOf("/");  // Find last forward slash
//        jscriptLog(myName+"Splitting path and filename gives path=\"" + temp.substr(0,i) + "\" and file =\"" + temp.substr(i+1) + "\"");
          try {
            result = comonFunctions.GetRandomFile(temp.substr(i+1), temp.substr(0,i) );
          }
          catch (err) {
            result = "";
          }
        }
        else {
          // No path portion, just a wildcard name so use it as is
//          jscriptLog(myName+"No path portion found so trying \"" + temp + "\" with folder \"\"");
          try {
            result = comonFunctions.GetRandomFile(temp, "");
          }
          catch (err) {
            result = "";
          }
        }
        // Did we get something valid?
        tries = tries + 1;
        done = true;
        if ( result.search(/thumbs\.db/i) != -1 ) {
          result = "";
          done = false;
        }
        else if ( comonFunctions.directoryExists(result) == true ) {
//          jscriptLog(myName + result + " is a folder");
          result = "";
          done = false;
        }
        if ( tries >= 10 ) {
          done = true;
//          jscriptLog(myName+" got " + result + " - max retries reached.");
        }
//        if ( ! done ) jscriptLog(myName+" got " + result + " - trying again (" + tries + ")");
      }
    }
    else {
      // Non wildcard file spec - use it as is
//     jscriptLog(myName+"Trying simple file spec \"" + temp + "\" with folder \"\"");
      // Test if the file exists
      if (comonFunctions.fileExists(temp) == true ) result = temp;
      else {
        result = "";
//        jscriptLog(myName+" file " + temp + " not found");
      }
    }
    result = "" + result;
//    jscriptLog(myName+"Final result is \"" + result + "\" type=" + typeof(result));
    return result;
}
 // ----------------------------------------------------------------------
  function handleTextFormats( text ) {
    // Page text can include special formatting characters like \n to insert a new line
    // or \h for <h2> style, or named styles.
    // This function handles all those codes and converts them to the proper html
    text = "" + text; // Force it to a string. Added Oct 2016 because I had
                      // an issue when passing an array element to this function
//    jscriptLog("handleTextFormats: Input is [" + text + "] " + text.length + " characters." );

    var i = 0;
    var inATag = false;
    var result = "";

    // Autoindent lines
    while (i < (text.length) && text.substr(i,1) == ">" ) {
      result += "&nbsp;&nbsp;"; i += 1;
    }

    // Now check for defined styles. Any line of page text can have an optional
    // style name as the first word like
    //    Style1 This text is style1
    //        OR
    //    >> Style1 This text is style1

    var s1 = i;
    while (s1 < text.length && text.charAt(s1) == " " ) s1++; // Skip spaces
      // s1 is now the start of the first word of the line
    var s2 = s1;
    while (s2 < text.length && text.charAt(s2) != " " ) s2++; // Skip non spaces to get the first word
      // s2 is the first space following the first word
//    jscriptLog("handleTextFormats: s1=" + s1 + ", s2=" + s2);
//    jscriptLog("handleTextFormats: Checking style [" + text.substring(s1,s2) +"]");
    var style = getStyle(text.substring(s1,s2));
    if ( style != "" ) { // A named style was found
//      jscriptLog("handleTextFormats: Found style " + text.substring(s1,s2));
      i = s2 + 1; // Skip over the style name. Otherwise i remains unchanged and
    }             // the word we just checked is included in the text

    // Now handle escaped characters and special characters like & \n, \{, etc.
    while ( i < text.length ) {
      if ( text.substr(i,1) == "\\" ) {  // Check for escaped characters
//        jscriptLog("handleTextFormats: Found a backslash followed by \"" + text.charAt(i+1) + "\"");
        if (text.charAt(i+1) == "n" ) { result += "<br/>"; i += 2; }
        else if ( text.charAt(i+1) == "h" ) { result += "<h2>"; i += 2; }
        else if ( text.charAt(i+1) == "p" ) { result += "<p>"; i += 2; }
        else if ( text.charAt(i+1) == "&" ) { result += "&amp;"; i += 2; }
        else if ( text.charAt(i+1) == " " ) { result += "&nbsp;"; i += 2; }
        else if ( text.charAt(i+1) == "<" ) { result += "&lt;"; i += 2; }
        else if ( text.charAt(i+1) == ">" ) { result += "&gt;"; i += 2; }
        else { result += text.charAt(i+1); i += 2;}
      }
      else if ( ! inATag && text.substr(i,1) == "/" ) {  // Check for custom end tag sequences
        // Unlike \ (backslash) only certain characters are supported.
        // If the following character does not match one of the ones listed
        // here the forward slash is ignored and just appears in the text.
        if ( text.charAt(i+1) == "h" ) { result += "</h2>"; i += 2; }
        else if ( text.charAt(i+1) == "p" ) { result += "</p>"; i += 2; }
        else { result += text.substr(i,1); i += 1;}
      }
      else if ( text.substr(i,1) == "<" ) { // Start of an html tag
        inATag = true;
        result += text.substr(i,1);
        i++;
      }
      else if ( text.substr(i,1) == ">" ) { // End of an html tag
        inATag = false;
        result += text.substr(i,1);
        i++;
      }
      else {  // Just copy the character
        result += text.substr(i,1);
        i++;
      }
    }

    // And finally, if a style was found earlier we wrap the entire line in it
    if ( style != "" ) {
      result = "<span style=\"" + style + "\">" + result + "</span>";
    }
//    jscriptLog("handleTextFormats: Result = [" + result +"]");
    return result;
  }
  // ----------------------------------------------------------------------
  function handleEscapedChars(text) {
    // In page text you can escape some special characters so that they appear
    // in the text rather than showing as html.
    // This is done using a backslash as in "\&" to get an ampersand.
    // This function replaces those escaped characters with their html equivalents.
//    jscriptLog("handleEscapedChars: Starting...");
    // Replaces angle brackets and round parentheses with their html equivalent &lt; and &gt;
    text = text.replace(/\\\\/g, "&#92;");
    text = text.replace(/\\\</g, "&lt;");
    text = text.replace(/\\\>/g, "&gt;");
    text = text.replace(/\\\(/g, "&#40;");
    text = text.replace(/\\\)/g, "&#41;");
//    text = text.replace(/\>/g, "&gt;");
//    text = text.replace(/\</g, "&lt;");
//    text = text.replace(/\(/g, "&#40;");
//    text = text.replace(/\)/g, "&#41;");
//    jscriptLog("handleEscapedChars: Ending with \"" + text + "\"");
    return text;
  }
  // ----------------------------------------------------------------------
  function replaceTags(text) {
    var myName = "replaceTags: ";
//    jscriptLog(myName+"Starting with text=\"" + text + "\"");
    // replaceTags will return a new string with all special tags replaced, or if
    // an error is encountered the returned value is undefined.
    // If an error is encountered reportScriptError() will already have been called
    // so you can just test for a return value of "undefined" and return up the chain.
    // For example
    //  command = replaceTags(command);
    //  if ( command == undefined ) return;
    // Otherwise you can assume everything is good with the world
    var errText = "";
    var temp;
    var tagStart; // Where the tag starts
    var argStart; // Where the arguments start
    var i;

    if ( text == "" ) return text;

    // First handle escaped characters
    text = handleEscapedChars(text);

    // Handle some special tags
    text = text.replace(/<engineversion>/gi,scriptEngineVersion());

    // Handle some common GuideMe settings.
    // We use square brackets temporarily so we can safely handle more general cases later
    text = text.replace(/<myname>/gi,"[span]myName[/span]");
    text = text.replace(/<mysubname>/gi,"[span]mySubName[/span]");
    text = text.replace(/<mistress1>/gi,"[span]mistress1[/span]");
    text = text.replace(/<mistress2>/gi,"[span]mistress2[/span]");
//    jscriptLog(myName+"After phase 1 text=\"" + text +"\"");

    // Now handle special tags like <random> and <ChooseFrom>

//    jscriptLog(myName+"Looking for <RandomFrom> in '" + text + "'" );

    // Check for <RandomFrom ...> tags
    tagStart = text.search(/<rand[omfr ,]+.*>/i); // Revised Sep 24/16
    while ( tagStart != -1 ) {
//      jscriptLog(myName+"Found a <random> tag in \"" + text + "\" at index " + tagStart);
      // Find the end of it extracting arguments as we go
//      jscriptLog(myName+"tagStart=" + tagStart);
      i = tagStart + 5; // Skip over "<rand"
      var args = []; // To collect the arguments
      // Find first whitespace, comma or end of tag (>)
      while ( text.substr(i,1) != " " && text.substr(i,1) != "," && text.substr(i,1) != ">") i = i + 1;
      if ( text.substr(i,1) == " " || text.substr(i,1) == "," ) i++;
      if (text.substr(i,1) == ">") {
//        reportScriptError("Invalid \\<Random\\> tag in \n" + escapeBrackets(text) + "\n");
        reportScriptError("Invalid \\<Random\\> tag in \n" + text + "\n");
        return;
      }
//      jscriptLog(myName+"first arg starts at i=" + i);
      var tagEnd = i; // Because it could be, but we don't know it is.
      while ( text.substr(i,1) != ">" ) { // So let's find out if it is
        start = i;
        end = i;
        i = i + 1;  // Point to next char
        // Find next delimiter
        while ( text.substr(i,1) != " " && text.substr(i,1) != "," && text.substr(i,1) != ">") {
          end = i;
          i = i + 1;
        }
        tagEnd = i;
//        jscriptLog(myName+"start = " + start + ", end=" + end + " arg = " + text.substring(start,end+1));

        args.push(text.substring(start,end+1));

        if ( text.substr(i,1) != ">" ) i++;
      }
//      jscriptLog(myName+"tagEnd=" + tagEnd);
//      jscriptLog(myName+"found " + args.length + " args");

      if ( args.length < 2 || args.length > 3 ) {
        reportScriptError("Invalid \\<Random\\> tag in \"" + text + "\"\n");
        return;
      }

      // Number of args okay. Are they valid?
      // Do variable substitution and see if we have valid numbers
      errText = "";
      if ( isNaN(getVariable(args[0])) ) errText += "&lt;Random&gt; tag has invalid argument: \"" + args[0] + "\" is not a number or variable is not defined\n";
      else args[0] = getVariable(args[0]);
      if ( isNaN(getVariable(args[1])) ) errText += "&lt;Random&gt; tag has invalid argument: \"" + args[1] + "\" is not a number or variable is not defined\n";
      else args[1] = getVariable(args[1]);
      if ( args.length == 3 ) {
        if ( isNaN(getVariable(args[2])) ) errText += "&lt;Random&gt; tag has invalid argument: \"" + args[2] + "\" is not a number or variable is not defined\n";
        else args[2] = getVariable(args[2]);
      }

//      jscriptLog(myName+"errText=\"" + errText + "\"");
      if ( errText != "" ) {  // Errors were detected
        reportScriptError( errText );
        return;
      }
      else {
        // Everything is okay so go do it
//        jscriptLog(myName+"Everything okay so far");
// Modified Jan 29/2017 for script engine version 1.5. I could no longer remember
// why I chose to have this round up, instead of just rounding. Simple rounding
// makes more sense and gives a more natural distribution of results.
// Also fixed so the rounding value could be a variable and added proper support
// for negative ranges. No idea what for, but while I was here....
//        i = RandomFrom(getVariable(args[0]),getVariable(args[1]));
//        if ( args.length == 3 ) { // Round up as specified
//         if(i > 0) return Math.ceil(i/args[2]) * args[2];
//         else if( i < 0) return Math.floor(i/args[2]) * args[2];
//         else return 0;
//        }

        i = RandomFrom(args[0],args[1]);
        if ( args.length == 3 ) { // Round as specified
          // It's a positive range like randomFrom(10,20,2)
          if(i > 0) {
            i = args[2] * Math.round(i/args[2]);
            if ( i < args[0] ) i = i + args[2]; // Go one mutiple up
            if ( i > args[1] ) i = i - args[2]; // Go one mutiple down
          }
          else if( i < 0) {
            // It's a negative range like randomFrom(-10,-20,2)
            i = args[2] * Math.round(i/args[2]);
            if ( i > args[0] ) i = i - args[2]; // Go one mutiple down
            if ( i < args[1] ) i = i + args[2]; // Go one mutiple up
          }
        }

        // And create the final result

//        jscriptLog(myName+"Original text = " + text);
//        jscriptLog(myName+"Random number is = " + i);
//        jscriptLog(myName+"tagStart = " + tagStart + ", tagEnd = " + tagEnd);
        text = text.substring(0,tagStart) + i + text.substr(tagEnd+1);
//        jscriptLog(myName+"Modified text = " + text);
      }

      // And see if there are any more to handle
      tagStart = text.search(/<rand[omfr ,]+.*>/i);
    }
    // ------    End processing of <RandomFrom ...> tags

    // Check for (x..y) type random number tags

//    jscriptLog(myName+"Looking for (n..m) tags in '" + text + "'" );

//    tagStart = text.search(/\(\S+\.\.\S+\)/);                   // Modified Jan 1, 2017 Look for (xxx..yyy) with brackets
    tagStart = text.search(/\([^\.\s]+\.\.[^\.\s]+\)/);               // Modified Jan 1, 2017 Look for (xxx..yyy) with brackets
//    jscriptLog(myName+"tagStart1 = " + tagStart );
    if ( tagStart == -1 ) tagStart = text.search(/[^\.\s]+\.\.[^\.\s]+/); // Modified Jan 1, 2017 If nothing found look for xxx..yyy without brackets
//    jscriptLog(myName+"tagStart2 = " + tagStart );

    while ( tagStart != -1 ) {
//      jscriptLog(myName+"tagStart = " + tagStart );
      // Found a string like something..somethingelse
      tagEnd = tagStart + 1;
      while ( tagEnd < text.length && text.charAt(tagEnd) != ")" && text.charAt(tagEnd) != " " && text.charAt(tagEnd) != "," ) tagEnd++;
      if ( tagEnd < text.length && text.charAt(tagEnd) == ")" ) tagEnd++; // Include closing bracket if present
//      jscriptLog(myName+"Found n..m at position " + tagStart + " to " + (tagEnd-1) );
      temp = text.substring(tagStart,tagEnd);
      temp = temp.replace("(", ""); // Remove brackets
      temp = temp.replace(")", "");
      var values = temp.split(/\.\./);  // Split it on the ..

//      jscriptLog(myName+"Got " + values[0] + " to " + values[1] );
      values[0] = getVariable(values[0]);
      values[1] = getVariable(values[1]);
      if ( isNaN(values[0]) || isNaN(values[1]) ) {
        // At least one of the arguments is not defined so just log a message and ignore the whole thing.
        tagStart = -1;  // And stop searching for more
        jscriptLog(myName+"Possible error in random tag " + temp);
        if ( isNaN(values[0]) ) {
          jscriptLog(myName+"" + values[0] + " is not defined");
//          errText += "Error in tag (" + temp + ") - " + values[0] + " is not defined\\n";
        }
        if ( isNaN(values[1]) ) {
          jscriptLog(myName+"" + values[1] + " is not defined");
//          errText += "Error in tag (" + temp + ") - " + values[1] + " is not defined\\n";
        }
//        if ( errText != "" ) {  // Errors were detected
//          reportScriptError( errText );
//          return;
//        }
//        jscriptLog(myName+"Text is \"" + text + "\"");
      }
      else {
        // Everything is okay so go do it
        temp = RandomFrom(values[0],values[1]);
//        jscriptLog(myName+"values[0]=" + values[0] + " values[1]=" + values[1] + " result = " + temp );
        text = text.substring(0,tagStart) + temp + text.substr(tagEnd);
        // Look for the next tag
//        tagStart = text.search(/\(*\S+\.\.\S+\)*/);
        tagStart = text.search(/\([^\.\s]+\.\.[^\.\s]+\)/);               // Modified Jan 1, 2017 Look for (xxx..yyy) with brackets
//        jscriptLog(myName+"tagStart3 = " + tagStart );
        if ( tagStart == -1 ) tagStart = text.search(/[^\.\s]+\.\.[^\.\s]+/); // Modified Jan 1, 2017 If nothing found look for xxx..yyy without brackets
//        jscriptLog(myName+"tagStart4 = " + tagStart );
      }
    }

    // Check for <ChooseFrom ...> tags
//    jscriptLog(myName+"Looking for ChooseFrom tags in '" + text + "'" );

    // Should we allow variables in a ChooseFrom?

    tagStart = text.search(/<\s*choose[from,\s]+/i)
//    jscriptLog(myName+"First chooseFrom tag starts at " + tagStart );
    while ( tagStart != -1) {
      argStart = tagStart + 1; // to Skip the "<"
      // Now skip over the word choosefrom to the first space or comma, or end of tag
      while (    argStart < text.length
              && text.charAt(argStart) != ">"
              && text.charAt(argStart) != " "
              && text.charAt(argStart) != ","
            ) argStart++;
      if ( text.charAt(argStart) == ">" ) {
        // Looks like we found an empty tag which is an error
        reportScriptError( "Bad Choosefrom tag encountered - possibly an empty tag in \n" + text);
        return;
      }
      else {
        argStart += 1;  // Skip the delimiter we found

        // Find the end of the tag
        tagEnd = argStart + 1;

        while ( tagEnd < text.length && text.charAt(tagEnd) != ">" ) tagEnd++;
//        jscriptLog(myName+"chooseFrom args found at chars " + argStart + " to " + tagEnd);
//        var options = text.match(/<\s*choosefrom\s+([^>]+)>/i);

        var options = text.substring(argStart,tagEnd);
        options = options.replace(/^\s+|\s+$/g,""); // Trim leading and trailing whitespace

        // We now use parsePageArgs() to split the options so it handles quoted strings with
        // embedded commas etc.
        options = parsePageArgs(options);

//        jscriptLog(myName+"<choosefrom> options=" + options);
        if ( options.length > 0 ) {
//          for ( i=0;i<options.length;i++ ) {
//            jscriptLog(myName+"option " + i + " = \"" + options[i] + "\"");
//          }

          // Now select an option
          i = RandomFrom(1,options.length)-1;
          text = text.slice(0,tagStart) + options[i] + text.slice(tagEnd+1);
          // jscriptLog(myName+"After tag replacement text=" + text);
        }
      }
      // Look for another one
      tagStart = text.search(/<\s*choose[from,\s]+/i)
    }
    // End processing of Choosefrom tags

//    jscriptLog(myName+"After phase 2 text='" + text +"'");

    // After all of the above is checked look for variable names enclosed in angle
    // brackets and replace that with the variable value

    // We test each instance of <word> to see if word is a defined variable or keyword.
    // If it is we replace it with the value of the variable or keyword.
    // Otherwise we assume it's some html tag the user has coded in the text
    // and we leave it alone

//    jscriptLog(myName+"--- Checking for special tags ---");
    tagStart = 0;

    while ( tagStart < text.length ) {
      while ( tagStart < text.length && text.charAt(tagStart) != "<" ) tagStart++;
      tagEnd = tagStart + 1;
      while ( tagEnd < text.length && text.charAt(tagEnd) != ">" ) tagEnd++;
      if ( tagEnd < text.length ) {
//        jscriptLog(myName+"tagStart=" + tagStart + ", tagEnd=" + tagEnd );
        var tag = text.substring(tagStart+1, tagEnd);

//        jscriptLog(myName+"testing tag \"" + tag +"\"");

        var tagValue = getVariable(tag);  // getVariable handles the special keywords for us as well

        if ( tagValue != undefined && tagValue != tag ) {
          // getVariable replaced it so it must have been a defined variable or keyword
          temp = "" + tagValue; // Forces it to be a string (getVariable returns a string,  float or int according to the value)
          text = text.substring(0,tagStart) + temp + text.substring(tagEnd+1);
          tagStart = tagStart + temp.length; // Skip over what we just inserted
        }
        else {
          // Must be some regular xml tag so just skip it
          tagStart = tagEnd + 1;  // Keep looking for next tag
//          Special keywords like RollxDice are handled by getVariable() so that
//          they work in both tags and expressions.
//          Don't handle them here.
        }
      }
      else {
        // We hit the end of the string without finding the end of the tag so this is messed up
        tagStart = tagEnd; // This will force the loop to end
      }
//      jscriptLog(myName+"text is now \"" + text + "\"" );
//      jscriptLog(myName+"new tagStart=" + tagStart );
    }
//    jscriptLog(myName+"After phase 3 text='" + text +"'");

    // Now finalize the span tag replacements
    text = text.replace(/\[span\]/g, "<span>");
    text = text.replace(/\[\/span\]/g, "</span>");

//    jscriptLog(myName+"Final text=\"" + text +"\"");

    return text;
  }

  // ----------------------------------------------------------------------
  function escapeBrackets(text) {
//    jscriptLog("escapeBrackets: Starting with \"" + text + "\"");
    // This function is only used for error reporting. It converts angle brackets
    // to their HTML code equivalents so that the command displays they way it
    // was written in the script rather than with the HTML interpreted.
    // It also supports a couple very basic escape sequences so we can format
    // error messages a bit.

    // Replace angle brackets with their html equivalent &lt; and &gt;
    text = text.replace(/\</g, "&lt;");
    text = text.replace(/\>/g, "&gt;");

    // Handle some basic codes for a little formatting in the error message
    // Replace "\n" with <br/>
    text = text.replace(/\n/g, "<br/>");
//    jscriptLog("escapeBrackets: Ending with \"" + text + "\"");
    return text;
  }

  // ----------------------------------------------------------------------
  function validateTags(text) {
    var myName = "validateTags: ";
    // Scans the given text for special tags and makes sure they are valid.
    // The return value is a null string if all is okay, or error text
    // describing any errors found.
    // Only the minimum checking is done and only for certain tags.
    // This is used when loading the script to detect tag errors at load time
    // instead of finding them halfway through testing the script.
    //
    var args; // The arguments to a tag
    var argStart;
    var tagStart;
    var tagEnd;
    var i;
    if ( text == "" ) return "";
    errText = "";

    // Check for and validate <RandomFrom ...> tags
    // Find a RandomFrom where the opening bracket is not escaped. Since we are testing
    // the character before the opening "<" tagStart will always be one character
    // before the actual start of the tag, unless the tag is at the very beginning
    // of the line, so we need to compensate for that.
    tagStart = text.search(/[^\\]<rand[^>]+>/i);
    if ( tagStart > 0 ) tagStart = tagStart + 1; // To skip the extra character. If it is exactly zero we don't change it.
//    if ( tagStart != -1 ) jscriptLog(myName+"Found a RandomFrom tag at char " + tagStart + " in \n" + text);
//          1         2         3         4         5         6         7         8         9         1         2         3         4         5
//0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789
//I want you to do <Choose 5,10,15,20> edges with <Choose 10,20,30> second breaks between them. Then stroke without edging for <Choose 5,10,15> minutes.
    while ( tagStart != -1) {
      argStart = tagStart + 1; // to Skip the "<"
      // Now skip over the word choosefrom to the first space or comma, or end of tag
      while (    argStart < text.length
              && text.charAt(argStart) != ">"
              && text.charAt(argStart) != " "
              && text.charAt(argStart) != ","
            ) argStart++;
      if ( text.charAt(argStart) == ">" ) {
        // Looks like we found an empty tag which is an error
        errText +=  "Empty or invalid  RandomFrom tag found in \n" + text + "\n";
        tagEnd = argStart + 1; // Skip the closing ">"
      }
      else {
        argStart += 1;  // Skip the delimiter we found

        // Find the end of the tag
        tagEnd = argStart + 1;

        while ( tagEnd < text.length && text.charAt(tagEnd) != ">" ) tagEnd++;
//        jscriptLog(myName+"RandomFrom args found at chars " + argStart + " to " + tagEnd);
        args = text.substring(argStart,tagEnd);
        args = args.replace(/^\s+|\s+$/g,""); // Trim leading and trailing whitespace
//        jscriptLog(myName+"<randomfrom> args=" + args);
        if ( args != "" && args.length > 1 ) {
          args = args.split(/,\s*/); // Split on commas
          if ( args.length <= 0 ) {
            errText += "Missing arguments in Randomfrom tag in \n" + text + "\n";
          }
          else if ( args.length < 2 ) {
            errText += "Too few arguments in Randomfrom tag in \n" + text + "\n";
          }
          else if ( args.length > 3 ) {
            errText += "Too many arguments in Randomfrom tag in \n" + text + "\n";
          }
        }
        tagEnd += 1; // Skip the closing ">"
      }
      // Look for another one
      i = text.substr(tagEnd).search(/[^\\]<rand[^>]+>/i);
      if ( i == -1 ) tagStart = -1;
      else tagStart = tagEnd + i + 1; // +1 so we skip the character before the "<" as well
//      jscriptLog(myName+"Next RandomFrom tag starts at " + tagStart );
    }

    // ----------------------------------
    // Check for <ChooseFrom ...> tags
    // ----------------------------------
//    jscriptLog(myName+"Looking for ChooseFrom tags in '" + text + "'" );

    // Find a choosefrom where the opening bracket is not escaped. Since we are testing
    // the character before the opening "<" tagStart will always be one character
    // before the actual start of the tag, unless the tag is at the very beginning
    // of the line, so we need to compensate for that.
    tagStart = text.search(/[^\\]<\s*choose[from,\s]+/i);
    if ( tagStart > 0 ) tagStart = tagStart + 1; // To skip the extra character. If it is exactly zero we don't change it.
//    if ( tagStart != -1 ) jscriptLog(myName+"Found a choosefrom tag at char " + tagStart + " in \n" + text);
//          1         2         3         4         5         6         7         8         9         1         2         3         4         5
//0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789
//I want you to do <Choose 5,10,15,20> edges with <Choose 10,20,30> second breaks between them. Then stroke without edging for <Choose 5,10,15> minutes.
    while ( tagStart != -1) {
      argStart = tagStart + 1; // to Skip the "<"
      // Now skip over the word choosefrom to the first space or comma, or end of tag
      while (    argStart < text.length
              && text.charAt(argStart) != ">"
              && text.charAt(argStart) != " "
              && text.charAt(argStart) != ","
            ) argStart++;
      if ( text.charAt(argStart) == ">" ) {
        // Looks like we found an empty tag which is an error
        errText +=  "Choosefrom tag has no arguments in \n" + text + "\n";
        tagEnd = argStart + 1;
//        return;
      }
      else {
        argStart += 1;  // Skip the delimiter we found

        // Find the end of the tag
        tagEnd = argStart + 1;

        while ( tagEnd < text.length && text.charAt(tagEnd) != ">" ) tagEnd++;
//        jscriptLog(myName+"chooseFrom args found at chars " + argStart + " to " + tagEnd);
//        var options = text.match(/<\s*choosefrom\s+([^>]+)>/i);
        var options = text.substring(argStart,tagEnd);
        options = options.replace(/^\s+|\s+$/g,""); // Trim leading and trailing whitespace
//        jscriptLog(myName+"<choosefrom> options=" + options);
        if ( options != "" && options.length > 1 ) {
          options = options.split(/,\s*/); // Split on commas only
          if ( options.length <= 0 ) {
            errText += "Missing arguments in Choosefrom tag in \n" + text + "\n";
          }
        }
        tagEnd += 1; // Skip the closing ">"
      }
      // Look for another one
      i = text.substr(tagEnd).search(/[^\\]<\s*choose[from,\s]+/i);
      if ( i == -1 ) tagStart = -1;
      else tagStart = tagEnd + i + 1; // +1 so we skip the character before the "<" as well
//      jscriptLog(myName+"Next chooseFrom tag starts at " + tagStart );
    }

//    jscriptLog(myName+"After phase 2 text=\"" + text +"\"");

    return errText;
  }

  // -----------------------------------------------------------------------
  function ExitCurrentScript() {
    // Exits the current script and reset script related variables.
    // Returns the ID of the recommended page to execute next.
    // This may be "runScript" to continue the previous script, or the page specfied
    // in the onScriptExit variable, or the default of "GuidemeScripts" whch displays
    // a list of available scripts for you to choose from.
    // Or of course you can override that with whatever you want.
    var scriptStack;
    var scriptData;
    var target;

    // Get the default target page
    target = scriptVars.get("onScriptExit"); // This may be null or undefined
    if ( target == undefined || target == "" ) {
      target = "GuidemeScripts";  // Default to GuidemeScripts but log a message
      jscriptLog("ExitCurrentScript: onScriptExit not set so defaulting to " + target);
    }

    // If a script is running, restore the environment saved when it started
    scriptData = scriptVars.get("scriptData");
    if ( scriptData != undefined && scriptData != "" ) {
      jscriptLog("ExitCurrentScript: Exiting script " + scriptData.filename);

      // Delete the waitingFor variable in case we were on the "Wait" page
      scriptVars.remove("waitingFor");

      // Restore key variables saved when this script started
      if ( scriptData.ImageFolder != undefined && scriptData.ImageFolder != "" ) scriptVars.put("ImageFolder", scriptData.ImageFolder);
      else scriptVars.remove("ImageFolder");

      if ( scriptData.AudioFolder != undefined && scriptData.AudioFolder != "" ) scriptVars.put("AudioFolder", scriptData.AudioFolder);
      else scriptVars.remove("AudioFolder");

      if ( scriptData.VideoFolder != undefined && scriptData.VideoFolder != "" ) scriptVars.put("VideoFolder", scriptData.VideoFolder);
      else scriptVars.remove("VideoFolder");

      jscriptLog("ExitCurrentScript: Restoring script path " + scriptData.ScriptPath );
      if ( scriptData.ScriptPath != undefined && scriptData.ScriptPath != "" ) scriptVars.put("ScriptPath", scriptData.ScriptPath);
      else scriptVars.remove("ScriptPath");

      if ( scriptData.onScriptExit != undefined && scriptData.onScriptExit != "" ) scriptVars.put("onScriptExit", scriptData.onScriptExit);
      else scriptVars.remove("onScriptExit");

      // Restore critical flags
      if ( scriptData.AutoLineBreaks == true )       comonFunctions.SetFlags("AutoLineBreaks", guide.getFlags());
      else comonFunctions.UnsetFlags("AutoLineBreaks", guide.getFlags());

      if ( scriptData.TextOnLeft == true )       comonFunctions.SetFlags("TextOnLeft", guide.getFlags());
      else comonFunctions.UnsetFlags("TextOnLeft", guide.getFlags());

      if ( scriptData.CardGameLayout1 == true )  comonFunctions.SetFlags("CardGameLayout1", guide.getFlags());
      else comonFunctions.UnsetFlags("CardGameLayout1", guide.getFlags());

      if ( scriptData.CardGameLayout2 == true )  comonFunctions.SetFlags("CardGameLayout2", guide.getFlags());
      else comonFunctions.UnsetFlags("CardGameLayout2", guide.getFlags());

      if ( scriptData.KeepDelays == true )       comonFunctions.SetFlags("KeepDelays", guide.getFlags());
      else comonFunctions.UnsetFlags("KeepDelays", guide.getFlags());

      if ( scriptData.KeepMetronomes == true )   comonFunctions.SetFlags("KeepMetronomes", guide.getFlags());
      else comonFunctions.UnsetFlags("KeepMetronomes", guide.getFlags());

      if ( scriptData.dontReplaceTags == true )  comonFunctions.SetFlags("dontReplaceTags", guide.getFlags());
      else comonFunctions.UnsetFlags("dontReplaceTags", guide.getFlags());
    }

    // Delete the scriptData object of the script that's exiting
    scriptVars.remove("scriptData");
    scriptVars.remove("ScriptLabel"); // In case we were on the ShowScriptLabels page

    // Now, if a scriptDataStack object exists and is nonzero length
    // then restore the previous script, otherwise just exit
    scriptStack = scriptVars.get("scriptDataStack");
//    jscriptLog("ExitCurrentScript: scriptStack is " + scriptStack + " type = " + typeof(scriptStack) );
    if ( scriptStack != undefined && scriptStack.length > 0 ) {
      //   Pop the next object from the stack and save it as scriptData
      scriptData = scriptStack.pop();
      jscriptLog("ExitCurrentScript: Exiting to previous script - " + scriptData.filename );

      // Restore the saved data
      scriptVars.put("scriptData",scriptData);

      // And continue the previous script
      target = "runScript";
    }
    else {
      // We're exiting the last (or only) script
      jscriptLog("ExitCurrentScript: Exiting final script...." );
      scriptVars.remove("scriptDataStack");
    }
    return target;
  }
  // -----------------------------------------------------------------------
  function ExitAllScripts() {
      jscriptLog("ExitAllScripts: Exiting all scripts ");
    // Abort all active scripts and reset script related variables.
    // Returns the ID of the recommended page to execute next.
    // This may be the page specfied in the onScriptExit variable, or the
    // default of "GuidemeScripts" whch displays a list of available scripts
    // for you to choose from.
    // And of course you can override that with whatever you want.
    var scriptStack;
    var scriptData;
    var target;

    // If a scriptDataStack object exists and is nonzero length
    // then restore settings from the first script called and just dump all the rest.
    scriptStack = scriptVars.get("scriptDataStack");
    if ( scriptStack != undefined && scriptStack.length > 0 ) {
      // Pop the first (oldest) object from the stack and save it as scriptData
      scriptData = scriptStack.shift(); // Get the first script called
    }
    else {
      // We're exiting the last (or only) script
      scriptData = scriptVars.get("scriptData");
    }

    // Restore the scriptVars saved when this script started
    if ( scriptData != undefined && scriptData != "" ) {
      jscriptLog("ExitAllScripts: Exiting script " + scriptData.filename);
      scriptVars.put("scriptData",scriptData);
      target = ExitCurrentScript();
    }
    else {
      jscriptLog("ExitAllScripts: Doesn't seem to be any scripts running. Resetting flags. " );
      // Get the default target page
      target = scriptVars.get("onScriptExit"); // This may be null or undefined
      if ( target == undefined || target == "" ) {
        target = "GuidemeScripts";  // Default to GuidemeScripts and log a message
        jscriptLog("ExitAllScripts: onScriptExit not set so defaulting to " + target);
      }
//      // If we didn't find a scriptData object or it was invalid should we just delete
//      // all script engine related variables and hope for the best??
//      // I think it makes sense to leave key ones like ImageFolder and
//      // ScriptPath set to whatever they are set to.
//      scriptVars.remove("AudioFolder");
//      scriptVars.remove("ImageFolder");
//      scriptVars.remove("onScriptExit");
//      scriptVars.remove("ScriptPath");
//      scriptVars.remove("VideoFolder");

      // Reset flags to standard defaults
      comonFunctions.UnsetFlags("AutoLineBreaks", guide.getFlags());
      comonFunctions.UnsetFlags("CardGameLayout1", guide.getFlags());
      comonFunctions.UnsetFlags("CardGameLayout2", guide.getFlags());
      comonFunctions.UnsetFlags("dontReplaceTags", guide.getFlags());
      comonFunctions.UnsetFlags("KeepDelays", guide.getFlags());
      comonFunctions.UnsetFlags("KeepMetronomes", guide.getFlags());
      comonFunctions.UnsetFlags("TextOnLeft", guide.getFlags());
    }

    // Delete the waitingFor variable in case we crashed on the "Wait" page
    scriptVars.remove("waitingFor");
    scriptVars.remove("ScriptLabel"); // In case we were on the ShowScriptLabels page

    // Delete the scriptData objects
    scriptVars.remove("scriptData");
    scriptVars.remove("scriptDataStack");

// Other variables we should maybe clean up
//  SlideShowLength = ""
//  SlideShowDelay = ""
//  SlideShowTarget = ""
//  SlideShowFolder = ""

    // And that does it.
    return target;
  }
  // -----------------------------------------------------------------------
  //    Additional suppport functions
  // -----------------------------------------------------------------------

      function SECountEdge(edges) {
          // Count edges done during the script, and if the tease provides a
          // CountEdge() function call it as well.
          if ( edges === undefined ) edges = 1;
          var n = parseInt(scriptVars.get("EdgeCount"));
          jscriptLog("SECountEdge(): EdgeCount = " + n);
          if ( n == undefined || n == "" ) n = 0;
          n = n + edges;
          scriptVars.put("EdgeCount", n);

          comonFunctions.SetFlags("UserEdged", guide.getFlags());

          // Also track todays edges while we're at
          var lastEdgeDate = parseInt(scriptVars.get("LastEdgeDate"));

          jscriptLog("SECountEdge(): lastEdgeDate=" + lastEdgeDate + ", theDate()=" + theDate());

          if ( lastEdgeDate == undefined || isNaN(lastEdgeDate) ) {
            // Initialize it
            lastEdgeDate = theDate();
            scriptVars.put("LastEdgeDate",""+lastEdgeDate);
            jscriptLog("SECountEdge(): Initialized lastEdgeDate to " + lastEdgeDate );
          }
          if ( lastEdgeDate == theDate() ) {
            n = parseInt(scriptVars.get("TodaysEdgeCount"));
            if ( n == undefined || n == "" ) n = 0;
            n = n + edges;
            scriptVars.put("TodaysEdgeCount", n);
            jscriptLog("SECountEdge():TodaysEdgeCount is now " + n );
          }
          else {
            // Must be our first edge(s) today
            scriptVars.put("LastEdgeDate",""+theDate());
            scriptVars.put("TodaysEdgeCount", edges);
            jscriptLog("SECountEdge():TodaysEdgeCount is now " + edges );
          }

          // This line is for integration into VDom - another project I have on the go
          // If it causes you problems delete it.
          // Otherwise you can safely ignore it.
          if ( typeof(CountEdge) === "function" ) CountEdge(edges);
      }

      // -----------------------------------------------------------------------
      function SESlideShowSetup(minDuration, maxDuration, delay, target) {
        // Set up a slideshow for a random number of minutes between minDuration
        // and maxDuration, changing the image every 'delay' seconds.
        // When done go to 'target'
//        jscriptLog("SESlideShowSetup(): minDuration=" + minDuration + ", maxDuration=" + maxDuration);
        minutes = RandomFrom(minDuration, maxDuration); // Duration in minutes
        duration = 60 * minutes; // Gives duration in seconds
  			scriptVars.put("SlideShowLength", duration);
  			scriptVars.put("SlideShowDelay", delay);
  			scriptVars.put("AfterSlideShow", target);
        scriptVars.put("SlideShowRemaining", duration);  // Initialize the time remaining

//        jscriptLog("SESlideShowSetup(): minutes=" + minutes + " (" + duration + " seconds), delay=" + delay + ", target=" + target);
      }

      // ---------------------------------------------------------------------------------
      //  Wrapper functions for use in scripts. These are also used by the main
      //  script engine code. They provide a simpler syntax for Guideme functions.
      // ---------------------------------------------------------------------------------

      // ---------------------------------------------------------------------------------
      function RandomFrom(min,max) {
        // Get a random number from min to max. This is just a wrapper
        // for comonFunctions.getRandom() that uses a simpler syntax
        if ( min == max ) return min;
    		var rand = Number(comonFunctions.getRandom("(" + parseInt(min) + ".." + parseInt(max) +")"));
    		return rand;
      }

      // ---------------------------------------------------------------------------------
      function isSet(flags) {
        // This is a wrapper function for comonFunctions.canShow() that tests
        // to see if one or more flags are set. See the documentation for
        // comonFunctions.canShow() for info on specifying the flags
        if ( comonFunctions.canShow(guide.getFlags(), flags, "" ) ) {
          return true;
        }
        return false;
      }

      // ---------------------------------------------------------------------------------
      function notSet(flags) {
        // This is a wrapper function for comonFunctions.canShow() that tests
        // to see if one or more flags are Not set. See the documentation for
        // comonFunctions.canShow() for info on specifying the flags
        if ( comonFunctions.canShow(guide.getFlags(), "", flags ) ) {
          return true;
        }
        return false;
      }

      // ---------------------------------------------------------------------------------
      function isDefined(variable) {
        // Tests whether or not the specified variable is defined and returns true or false.
        // The variable name is case sensitive
//        jscriptLog("isDefined: Testing " + variable);
        var result;

//        var temp = scriptVars.get(variable);
//        jscriptLog("isDefined: temp = " + temp);
//        if ( temp == undefined || temp == "" ) result = false;
//        else result = true;

        // Updated Oct 2017 to use getVariable() to support
        // Arg1-Argn when passing arguments to subroutines
        var temp = getVariable(variable);
        if ( temp == variable || temp == "" ) result = false;
        else result = true;

//        jscriptLog("isDefined: result is " + result);
        return result;
      }

      // The notDefined() function is recognized by the parser and replaced with the
      // javascript equivalent ! isDefined()

      // ---------------------------------------------------------------------------------
      function pageExists( target ) {
        // Returns true if the target page exists in this guide, or false otherwise
        var pageExists = guide.getChapters().get("default").getPages().containsKey(target);
        if ( pageExists ) return true;
        return false;
      }

      // ---------------------------------------------------------------------------------
      //  Functions for simulating dice rolls. These functions roll the dice and return
      // the number rolled, and set variable NumberRolled as well for later reference.
      // ---------------------------------------------------------------------------------
    	function RollOneDice() {
        var result = RandomFrom(1,6);
    //    scriptVars.put("vDiceComboRolled", "1-" + result);
    		scriptVars.put("NumberRolled", result);
        jscriptLog("Roll1Dice: NumberRolled=" + result);
        return result;
    	}
      // ---------------------------------------------------------------------------------
    	function RollTwoDice() {
        // All the possible combinations
        var theNumbers = [0,2,3,4,5,6,7,3,4,5,6,7,8,4,5,6,7,8,9,5,6,7,8,9,10,6,7,8,9,10,11,7,8,9,10,11,12];
    //    var theCombo = ["", "1+1", "1+2", "1+3", "1+4", "1+5", "1+6",
    //                        "2+1",
        var result = RandomFrom(1,36);
        result = theNumbers[result];
    		scriptVars.put("NumberRolled", result);
        jscriptLog("RollTwoDice: NumberRolled=" + result);
        return result;
      }

      // ---------------------------------------------------------------------------------
    	function RollThreeDice() {
        var theNumbers = [0,3,4,5,6,7,8,4,5,6,7,8,9,5,6,7,8,9,10,6,7,8,9,10,11,7,8,9,10,11,12,8,9,10,11,12,13,4,5,6,7,8,9,5,6,7,8,9,10,6,7,8,9,10,11,7,8,9,10,11,12,8,9,10,11,12,13,9,10,11,12,13,14,5,6,7,8,9,10,6,7,8,9,10,11,7,8,9,10,11,12,8,9,10,11,12,13,9,10,11,12,13,14,10,11,12,13,14,15,6,7,8,9,10,11,7,8,9,10,11,12,8,9,10,11,12,13,9,10,11,12,13,14,10,11,12,13,14,15,11,12,13,14,15,16,7,8,9,10,11,12,8,9,10,11,12,13,9,10,11,12,13,14,10,11,12,13,14,15,11,12,13,14,15,16,12,13,14,15,16,17,8,9,10,11,12,13,9,10,11,12,13,14,10,11,12,13,14,15,11,12,13,14,15,16,12,13,14,15,16,17,13,14,15,16,17,18];
        var result = RandomFrom(1,216);
    //  	scriptVars.put("vDiceComboRolled", "3-" + result);
        result = theNumbers[result];
    		scriptVars.put("NumberRolled", result);
        jscriptLog("Roll3Dice: NumberRolled=" + result);
        return result;
    	}

      // -----------------------------------------------------------------------
      function theDate(date) {
        // Returns the date as an integer in the form yyyymmdd
        // If no date is given todays date is returned.
        if ( date == undefined || date == "" ) when = new Date();
        else when = new Date(date);

        var temp = when.getFullYear() * 10000;
        temp = temp + (when.getMonth()+1)*100 + when.getDate();
        return temp;
      }

      // -----------------------------------------------------------------------
      function theTime(date) {
        // Returns the time as an integer in the form hhmm
        // If no date is given the current time is returned
        if ( date == undefined || date == "" ) when = new Date();
        else when = new Date(date);

        var temp = when.getHours() * 100;
        temp = temp + when.getMinutes();
        return temp;
      }
      // -----------------------------------------------------------------------
      function secondsUntil(targetDate) {
        // Returns the number of seconds until a future date/time.
        // Useful for calculating delays when you want things to happen around
        // a certain time of day.
        //
        var d = new Date();
        var vNow = d.getTime();
        d.setTime(targetDate);
        var vThen = d.getTime();
        var elapsed = vThen - vNow;
        return Math.floor(elapsed/1000);
      }
]]>
</GlobalJavascript>